/*=============================================================================
	Module Name:
			ConRedir.h

	Abstract:
			Console application standard handle redirector

    License:
            GNU General Public License (GPL)

	Revision History:
			* Nov 20, 2004:  Derived from QuickWin by Attila Afra
            * Feb 27, 2005:  Custom priority class
            * Mar 25, 2005:  Line based output events

------------------------------------------------------------------------------
	Copyright  2004-2005 Attila T. Afra. All rights reserved.
=============================================================================*/

#ifndef ConRedirH
#define ConRedirH

// Includes
#include <windows.h>

// CConRedir class
class CConRedir
{
public:
    // Construction
	CConRedir();
	~CConRedir();

    // Operations
	BOOL StartChildProcess(LPCSTR lpszCmdLine, DWORD dwPriorityClass, BOOL bShowChildWindow = FALSE);
	BOOL IsChildRunning() const;
    void SetPriorityClass(DWORD dwPriorityClass);
	void TerminateChildProcess();
	BOOL WriteChildStdIn(LPCSTR lpszInput);

protected:
	HANDLE m_hExitEvent;

	// Child input(stdin) & output(stdout, stderr) pipes
	HANDLE m_hStdIn, m_hStdOut, m_hStdErr;
	// Parent output(stdin) & input(stdout) pipe
	HANDLE m_hStdInWrite, m_hStdOutRead, m_hStdErrRead;
	// stdout, stderr write threads
	HANDLE m_hStdOutThread, m_hStdErrThread;
	// Monitoring thread
	HANDLE m_hProcessThread;
	// Child process handle
	HANDLE m_hChildProcess;
    // Line buffers
    CHAR *m_StdOutLineBuf;
    DWORD m_StdOutLineBufSize;
    CHAR *m_StdErrLineBuf;
    DWORD m_StdErrLineBufSize;

	HANDLE PrepAndLaunchRedirectedChild(LPCSTR lpszCmdLine,
		HANDLE hStdOut, HANDLE hStdIn, HANDLE hStdErr,
		DWORD dwPriorityClass, BOOL bShowChildWindow);

	static BOOL m_bRunThread;

	static int staticStdOutThread(CConRedir *pConRedir)
    {
        return pConRedir->StdOutThread(pConRedir->m_hStdOutRead);
    }

	static int staticStdErrThread(CConRedir *pConRedir)
    {
        return pConRedir->StdErrThread(pConRedir->m_hStdErrRead);
    }

	static int staticProcessThread(CConRedir *pConRedir)
    {
        return pConRedir->ProcessThread();
    }

	int StdOutThread(HANDLE hStdOutRead);
	int StdErrThread(HANDLE hStdErrRead);
	int ProcessThread();

public:
    // Events
	virtual void OnChildStarted(LPCSTR lpszCmdLine) {};
	//virtual void OnChildStdOutWrite(LPCSTR lpszOutput) {};
    virtual void OnChildStdOutWriteLn(LPCSTR lpszOutput) {};
	//virtual void OnChildStdErrWrite(LPCSTR lpszOutput) {};
    virtual void OnChildStdErrWriteLn(LPCSTR lpszOutput) {};
	virtual void OnChildTerminate() {};
};

#endif // ConRedirH
