/*=============================================================================
	Module Name:
			DVDSum.cpp

	Abstract:
			DVD-Video fast checksum generator

    License:
            GNU General Public License (GPL)

	Revision History:
			* Mar  4, 2005:  Created by Attila Afra

------------------------------------------------------------------------------
	Copyright  2005 Attila T. Afra. All rights reserved.
=============================================================================*/

// Borland C++ compiler only
#ifndef __BORLANDC__
#error Sorry, Borland C++ required!
#endif

// Includes
#include <vcl.h>
#include <stdio.h>
#include "chksum.h"

// Constants
#define CRC_MASK        0xFFFFFFFF
#define MAX_IFO_SIZE    16 * 1024 * 1024    // 16 MB

// Get the size of a file
__int64 _FileSize(AnsiString FileName)
{
    int Handle;
    __int64 Size;

    if ((Handle = FileOpen(FileName, fmOpenRead | fmShareDenyNone)) == -1)
        return -1;

    Size = FileSeek(Handle, __int64(0), 2);
    FileClose(Handle);

    return Size;
}

// IFO checksum generation
bool GenerateIFOSum(unsigned long& CRC, char *Filename)
{
    FILE *File;
    int FileSize;
    char *Data;

    // Open file
    if ((File = fopen(Filename, "rb")) == NULL)
        return false;

    // Get filesize
    fseek(File, 0, SEEK_END);
    FileSize = ftell(File);
    if (FileSize > MAX_IFO_SIZE)
    {
        fclose(File);
        return false;
    }
    fseek(File, 0, SEEK_SET);

    // Read data
    Data = new char[FileSize];
    fread(Data, FileSize, 1, File);
    fclose(File);

    // Generate checksum
    CRC = crc32(CRC, Data, FileSize);

    // Free memory
    delete[] Data;
    return true;
}

// Main function
bool GenerateDVDSum(char *Path, unsigned long& CRC)
{
    AnsiString VTSPath = Path, File;
    TStringList *List = new TStringList;
    int i, Major, Minor;
    __int64 Size;
    TSearchRec sr;
    int Result;

    // Initialization
    CRC = CRC_MASK;
    List->Sorted = true;

    // Adjust path
    while (VTSPath.Length() && VTSPath[VTSPath.Length()] == '\\')
        VTSPath.SetLength(VTSPath.Length() - 1);

    VTSPath += "\\";

    // Process VIDEO_TS.IFO file
    List->Clear();

    File = VTSPath + "VIDEO_TS.IFO";
    if (FileExists(File) && !GenerateIFOSum(CRC, File.c_str()))
    {
        delete List;
        return false;
    }

    // Process VTS_??_0.IFO files
    Result = FindFirst(VTSPath + "VTS_??_0.IFO", faAnyFile ^ faDirectory ^ faVolumeID, sr);

    while (!Result)
    {
        File = sr.Name.UpperCase();
        if (sscanf(File.c_str(), "VTS_%d_0.IFO", &Major) == 1)
            List->Add(File);

        // Next IFO
        Result = FindNext(sr);
    }

    FindClose(sr);

    for (i = 0; i < List->Count; i++)
    {
        sscanf(List->Strings[i].c_str(), "VTS_%d_0.IFO", &Major);
        CRC = crc32(CRC, (const char *)&Major, 1);
        if (!GenerateIFOSum(CRC, (VTSPath + List->Strings[i]).c_str()))
        {
            delete List;
            return false;
        }
    }

    // Process VIDEO_TS.VOB file
    List->Clear();

    File = VTSPath + "VIDEO_TS.VOB";
    if (FileExists(File) && (Size = _FileSize(File)) == -1)
    {
        delete List;
        return false;
    }
    CRC = crc32(CRC, (const char *)&Size, sizeof(Size));

    // Process VTS_??_?.VOB files
    Result = FindFirst(VTSPath + "VTS_??_?.VOB", faAnyFile ^ faDirectory ^ faVolumeID, sr);

    while (!Result)
    {
        File = sr.Name.UpperCase();
        if (sscanf(File.c_str(), "VTS_%d_%d.VOB", &Major, &Minor) == 2)
            List->Add(File);

        // Next VOB
        Result = FindNext(sr);
    }

    FindClose(sr);

    for (i = 0; i < List->Count; i++)
    {
        sscanf(List->Strings[i].c_str(), "VTS_%d_%d.VOB", &Major, &Minor);
        CRC = crc32(CRC, (const char *)&Major, 1);
        CRC = crc32(CRC, (const char *)&Minor, 1);
        if ((Size = _FileSize(VTSPath + List->Strings[i])) == -1)
        {
            delete List;
            return false;
        }
        CRC = crc32(CRC, (const char *)&Size, sizeof(Size));
    }

    delete List;
    return true;
}
