/*=============================================================================
    Project Name:
            FilmShrink

	Module Name:
			FSKMenu.h

	Abstract:
			FilmShrink main menu form

    License:
            GNU General Public License (GPL)

	Revision History:
			* Aug  9, 2004:  Created by Attila Afra
            * Jan 17, 2005:  Minor changes
            * Jan 23, 2005:  Help extensions
            * Feb  6, 2005:  Better time estimation

------------------------------------------------------------------------------
	Copyright  2004-2005 Attila T. Afra. All rights reserved.
=============================================================================*/

#ifndef FSKMenuH
#define FSKMenuH

// Includes
#include <stdio.h>

#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include <ComCtrls.hpp>
#include <ToolWin.hpp>
#include <ImgList.hpp>
#include <Buttons.hpp>
#include "TB2Dock.hpp"
#include "TB2Toolbar.hpp"
#include "TBX.hpp"
#include "TB2Item.hpp"
#include "ThemeMgr.hpp"

#include "MainMacros.h"
#include "ConRedir.h"
#include <ExtCtrls.hpp>

// EFSKJobStatus enum
enum EFSKJobStatus
{
    JS_Pending,
    JS_Working,
    JS_Done,
    JS_Invalid
};

// EFSKJobInfoIndex enum
enum EFSKJobInfoIndex
{
    JI_Name,
    JI_Date,
    JI_Vol,
    JI_Time,
    JI_TimeLeft,
    JI_Path
};

/*-----------------------------------------------------------------------------
    TFSKMenuForm class
-----------------------------------------------------------------------------*/

class TFSKMenuForm : public TForm
{
    // Friends
    friend class TFSKNewForm;
    friend class TFSKEncoder;

__published:
    // IDE-managed Components
    TListView *JobList;
    TImageList *MenuImageList;
    TTBXDock *Dock;
    TTBXToolbar *Toolbar;
    TTBXItem *StartItem;
    TTBXItem *StartAllItem;
    TTBXItem *AbortItem;
    TTBXSeparatorItem *SeparatorItem1;
    TTBXItem *NewItem;
    TTBXItem *AddItem;
    TTBXItem *DelItem;
    TTBXSeparatorItem *SeparatorItem2;
    TTBXItem *UpItem;
    TTBXItem *DownItem;
    TTBXSeparatorItem *SeparatorItem3;
    TTBXSubmenuItem *HelpSubmenuItem;
    TTBXItem *HelpAboutItem;
    TTBXItem *HelpLicenseItem;
    TTBXItem *HelpReadmeItem;
    TImageList *StatusImageList;
    TThemeManager *ThemeManager;
    TGroupBox *ProgBox;
    TGroupBox *ShutBox;
    TProgressBar *ProgVolBar;
    TProgressBar *ProgTotalBar;
    TLabel *ProgVolLabel1;
    TLabel *ProgVolLabel2;
    TLabel *ProgTotalLabel1;
    TLabel *ProgTotalLabel2;
    TLabel *ProgLabel1;
    TLabel *ProgLabel3;
    TLabel *ProgLabel4;
    TLabel *ProgLabel2;
    TLabel *ProgJobLabel;
    TLabel *ProgDateLabel;
    TLabel *ProgTimeLabel;
    TLabel *ProgLeftLabel;
    TTimer *Timer;
    TCheckBox *ShutCheck;
    TRadioButton *ShutAllRadio;
    TRadioButton *ShutJobRadio;
    TRadioButton *ShutVolRadio;
    TEdit *ShutJobEdit;
    TEdit *ShutVolEdit;
    TUpDown *ShutJobUpDown;
    TUpDown *ShutVolUpDown;
    TComboBox *PriorityCombo;
    TLabel *PriorityLabel;

    // Events
    void __fastcall FormShow(TObject *Sender);
    void __fastcall FormHide(TObject *Sender);
    void __fastcall FormClose(TObject *Sender, TCloseAction &Action);

    void __fastcall JobListSelectItem(TObject *Sender, TListItem *Item, bool Selected);

    void __fastcall StartItemClick(TObject *Sender);
    void __fastcall StartAllItemClick(TObject *Sender);
    void __fastcall AbortItemClick(TObject *Sender);
    void __fastcall NewItemClick(TObject *Sender);
    void __fastcall AddItemClick(TObject *Sender);
    void __fastcall DelItemClick(TObject *Sender);
    void __fastcall UpItemClick(TObject *Sender);
    void __fastcall DownItemClick(TObject *Sender);
    void __fastcall HelpAboutItemClick(TObject *Sender);
    void __fastcall HelpReadmeItemClick(TObject *Sender);
    void __fastcall HelpLicenseItemClick(TObject *Sender);

    void __fastcall ShutCheckClick(TObject *Sender);
    void __fastcall ShutAllRadioClick(TObject *Sender);
    void __fastcall ShutJobRadioClick(TObject *Sender);
    void __fastcall ShutVolRadioClick(TObject *Sender);

    void __fastcall TimerTimer(TObject *Sender);
    void __fastcall FormActivate(TObject *Sender);
    void __fastcall PriorityComboChange(TObject *Sender);

// User declarations
private:
    MAIN_WNDPROC_PROTO  // event handler

    // Variables
    AnsiString   FormCaption;                   // caption of this form (in normal mode)
    int          PendingJobCount;               // number of valid jobs pending
    bool         IsWorking, IsBatch;            // job/batch encoding flag
    TListItem   *CurJobItem;                    // current job item
    TFSKEncoder *Encoder;                       // encoder class
    TDateTime    StartTime;                     // start date/time
    int          TimeOffset;                    // time offset in seconds (from previous encoding session)
    int          TimeElapsed;                   // time elapsed in seconds
    AnsiString   LastTime, LastTimeLeft;        // timestamps at the end of the last volume
    int          ShutJobMax, ShutVolMax;        // maximum number of jobs and volumes available (when idle)
    int          PrevPriority;                  // previous selected priority
    bool         JobListSelectItemUpdateOnly;

    // Job operations
    void LoadJobList();
    void SaveJobList();

    void StartJob(int JobIndex);
    void AbortJob();

    bool AddJob(AnsiString Path, bool &Exists);
    void SwapJobs(int i, int j);
    bool WriteJobInfo(AnsiString Filename, TListItem *Item);

    void StartJobVol(int Vol, int VolTotal);
    void _EndJobVol(int Vol, int VolTotal);
    void _AbortJobVol(bool Error, AnsiString Message = NULL);

    // Timer
    void StartTimer();
    void StopTimer();

    // Other
    void ResetProgInfo();
    void UpdateShutValues();
    void Shutdown();
    DWORD GetPriority();

    void RegError();
    bool DeleteKeyTree(AnsiString Key);

public:
    __fastcall TFSKMenuForm(TComponent* Owner);
    __fastcall ~TFSKMenuForm();
};

// External MenuForm declaration
extern PACKAGE TFSKMenuForm *FSKMenuForm;

/*-----------------------------------------------------------------------------
    TFSKEncoder class
-----------------------------------------------------------------------------*/

class TFSKEncoder : public CConRedir
{
private:
    // Variables
    int  CurVol, NumVol;                     // current volume / number of volumes
    bool IsTwoPass;                          // 2-pass encoding?
    int  EncSpeed, Pass1Speed, TotalSpeed;   // relative encoding speeds
    int  PrevProgress, PrevPass1Progress;    // previous progress checkpoint

    bool IsActive, IsDone, IsAborted, IsKilled; // is the encoding process active/done/aborted/selfkilled ?
    int  PID;                                   // process id

    // Form control methods
    void SetProgress(double Progress, bool Estimate);

public:
    // Public variables
    double RefTotalProgress;    // reference total progress
    int    RefTimeElapsed;      // reference elapsed time (in seconds)

    // Control
    bool Start(AnsiString EncoderPath, AnsiString JobName, int Vol, int VolTotal, DWORD Priority);
    void Stop(bool Error);

    // Info
    bool IsWorking()
    {
        return IsActive;
    }

    // Events
    virtual void OnChildStdOutWriteLn(LPCSTR lpszOutput);
    virtual void OnChildTerminate();
};

#endif
