/*=============================================================================
    Project Name:
            FilmShrink

	Module Name:
			FSKNew.cpp

	Abstract:
			FilmShrink "New Job" wizard

    License:
            GNU General Public License (GPL)

	Revision History:
			* Aug  1, 2004:  Created by Attila Afra
            * Jan  5, 2005:  Subpicture support
            * Jan 27, 2005:  Bugfixes
            * Feb  5, 2005:  Major changes
            * Feb  6, 2005:  Bitrate calculation bug fixed
            * Feb  7, 2005:  Preview optimizations
            * Mar 29, 2005:  Improved TEdit->Text verification
            * Apr  1, 2005:  Minor GUI bug fixed

------------------------------------------------------------------------------
	Copyright  2004-2005 Attila T. Afra. All rights reserved.
=============================================================================*/

// Warnings
#pragma warn -8060  // Possibly incorrect assignment
#pragma warn -8004  // ... is assigned a value that is never used

#include <vcl.h>
#pragma hdrstop

// Includes
#include <ctype.h>
#include <stdio.h>
#include <math.h>
#include <DateUtils.hpp>

#include "m2preview.h"
#include "langcode.h"

#include "Shell.h"
#include "FSKGlobal.h"
#include "FSKMenu.h"
#include "FSKNewWait.h"
#include "FSKNew.h"
#include "FSKNewThreads.h"
#include "FSKNewFilters.h"
#include "FSKNewCodec.h"
#include "FSKNewCrop.h"

// VCL stuff
#pragma package(smart_init)
#pragma resource "*.dfm"

// FSKNewForm definition
TFSKNewForm *FSKNewForm;

/*-----------------------------------------------------------------------------
    Globals
-----------------------------------------------------------------------------*/

#define PATH_LIMIT           (MAX_PATH - 4)    // max path length

// Text
#define MSG_CLOSE_WIZARD     "Do you really want to close this wizard?"
#define MSG_DISCARD_ANALYSIS "The result of the analysis will be discarded. Are you sure you want to do this?"
#define MSG_PREVIEW_ERROR    "Unable to preview selected title! The DVD is probably damaged!"

#define TXT_NO_TITLE         ""

/*=============================================================================
    TFSKNewForm
=============================================================================*/

/*-----------------------------------------------------------------------------
    TFSKNewForm >> Constructors
-----------------------------------------------------------------------------*/

// Default constructor
__fastcall TFSKNewForm::TFSKNewForm(TComponent* Owner) : TForm(Owner)
{
    int             i;                  // counters
    unsigned short  Year, Month, Day;   // current date
    TListItem      *Item;               // list item

    // Initialize values
    AnalyzeThread  = 0;
    S1_IsFirstTime = true;
    S1_PrevPathLen = S1_PrevSrcPathLen = 0;
    S2_Finished    = S2_Active         = false;
    S3_PrevVTS     = S3_PrevPGC = S3_PrevAngle = -1;

    // Initialize volume size presets
    VolCombo->ItemsEx->AddItem("Custom", 0, 0, 0, 0, NULL);

    i = 0;
    while (FSKVolumeSize[i].Size)
    {
        VolCombo->ItemsEx->AddItem(FSKVolumeSize[i].Name,
                                       FSKVolumeSize[i].Type,
                                       FSKVolumeSize[i].Type,
                                       FSKVolumeSize[i].Type,
                                       0,
                                       NULL);

        i++;
    }

    // Calculate top/height of preview image (Step 3) in 4:3 and 16:9 modes
    S3_Pre43Height  = PreviewImage->Height;
    S3_Pre43Top     = PreviewImage->Top;
    S3_Pre169Height = PreviewImage->Width * 9 / 16;
    S3_Pre169Top    = S3_Pre43Top + (S3_Pre43Height - S3_Pre169Height) / 2;

    // Create preview frame bitmap (Step 3)
    S3_PreviewFrame = new Graphics::TBitmap();
    S3_PreviewFrame->PixelFormat = pf24bit;

    // Initialize info fields (Step 4)
    DecodeDate(Now(), Year, Month, Day);
    InfoCopyEdit->Text = "(C) " + IntToStr(Year);

    // Initialize video info fields (Step 4)
    Item = VideoInfoList->Items->Add();
    Item->Caption = "System"; Item->SubItems->Add("");

    Item = VideoInfoList->Items->Add();
    Item->Caption = "Resolution"; Item->SubItems->Add("");

    Item = VideoInfoList->Items->Add();
    Item->Caption = "Display"; Item->SubItems->Add("");

    Item = VideoInfoList->Items->Add();
    Item->Caption = "Frame rate"; Item->SubItems->Add("");

    Item = VideoInfoList->Items->Add();
    Item->Caption = "Frame type"; Item->SubItems->Add("");

    Item = VideoInfoList->Items->Add();
    Item->Caption = "Duration"; Item->SubItems->Add("");

    // Initialize image transparency
    VideoResImage->Picture->Bitmap->TransparentColor = clFuchsia;
    VideoCropImage->Picture->Bitmap->TransparentColor = clFuchsia;

    // Misc
    PreviewButton->Visible = false;
}

// Destructor
__fastcall TFSKNewForm::~TFSKNewForm()
{
    // Free VTS table
    for (int i = 0; i < 100; i++)
        if (TitleInfo[i].PGCNum)
        {
            for (int j = 1; j <= TitleInfo[i].PGCNum; j++)
            {
                if (TitleInfo[i].PGC[j].AudioNum)
                    delete[] TitleInfo[i].PGC[j].Audio;

                if (TitleInfo[i].PGC[j].SubpicNum)
                    delete[] TitleInfo[i].PGC[j].Subpic;
            }

            delete[] TitleInfo[i].PGC;
        }

    // Free preview frame bitmap
    delete S3_PreviewFrame;
}

/*-----------------------------------------------------------------------------
    TFSKNewForm >> Events
-----------------------------------------------------------------------------*/

// TFSKNewForm::WndProc
MAIN_WNDPROC(TFSKNewForm, FSKMainForm)


void __fastcall TFSKNewForm::FormActivate(TObject *Sender)
{
    FSKActiveForm = this;
}

void __fastcall TFSKNewForm::FormClose(TObject *Sender, TCloseAction &Action)
{
    int DoClose;

    // Terminate analyze thread if active
    if (S2_Active)
        DoClose = S2_Abort(false);
    else
        DoClose = Application->MessageBoxA(MSG_CLOSE_WIZARD, FSK_PRODUCT_NAME, MB_YESNO | MB_ICONEXCLAMATION) == mrYes;

    if (DoClose)
    {
        // Close video file if open
        if (S3_Selected)
            m2PreviewClose();

        // Remove job directory if exists
        if (DirectoryExists(JobRoot))
            RemoveDirTree(JobRoot);

        Action = caFree;
        if (FSKTermFlag)
            FSKMenuForm->Close();
        else
            FSKMenuForm->Enabled = true;
    }
    else
    {
        FSKTermFlag = false;
        Action = caNone;
    }
}

void __fastcall TFSKNewForm::FormShow(TObject *Sender)
{
    MAIN_SHOW_INIT(FSKMainForm);

    // Set cursor to default
    Screen->Cursor = crDefault;

    // Start with step 1
    Notebook->PageIndex = 0;
    S1_Begin();
}

void __fastcall TFSKNewForm::NextButtonClick(TObject *Sender)
{
    // Jump to the next step
    switch (Notebook->PageIndex)
    {
    case 0:
        if (S1_End(true))
        {
            S2_Begin();
            break;
        }
        else
            return;

    case 1:
        if (S2_End(true))
        {
            S3_Begin();
            break;
        }
        else
            return;

    case 2:
        if (S3_End(true))
        {
            S4_Begin();
            break;
        }
        else
            return;

    case 3:
        if (S4_End(true))
            break;
        else
            return;

    default:
        return;
    }
}

void __fastcall TFSKNewForm::BackButtonClick(TObject *Sender)
{
    // Jump to the prevous step
    switch (Notebook->PageIndex)
    {
    case 1:
        if (S2_End(false))
        {
            S1_Begin();
            break;
        }
        else
            return;

    case 2:
        if (S3_End(false))
        {
            S1_Begin();
            break;
        }
        else
            return;

    case 3:
        if (S4_End(false))
        {
            S3_Begin();
            break;
        }
        else
            return;

    default:
        return;
    }
}

void __fastcall TFSKNewForm::CancelButtonClick(TObject *Sender)
{
    Close();
}

/*=============================================================================
    TFSKNewForm >> Step 1
=============================================================================*/

/*-----------------------------------------------------------------------------
    TFSKNewForm >> Step 1 >> Events
-----------------------------------------------------------------------------*/

void __fastcall TFSKNewForm::NameEditKeyPress(TObject *Sender, char &Key)
{
    // Ignore invalid characters
    if ((Key != 8) /* Backspace */ && !isalnum(Key) && Key != ' ' && Key != '-' && Key != '_')
    {
        Key = 0; // No key pressed
        MessageBeep(MB_OK);
    }
}

void __fastcall TFSKNewForm::NameEditChange(TObject *Sender)
{
    bool IsEmpty;

    IsEmpty = NameEdit->Text.IsEmpty() || LocEdit->Text.IsEmpty();
    if (!IsEmpty && S1_UpdatePath())
    {
        if (S1_VerifySrcPath())
        {
            NextButton->Enabled = true;
            return;
        }
    }
    else
        S1_HidePath();

    NextButton->Enabled = false;
}

void __fastcall TFSKNewForm::NameEditExit(TObject *Sender)
{
    if (!NameEdit->Text.Length())
        return;

    for (int i = 1; i <= NameEdit->Text.Length(); i++)
        if (!isalnum(NameEdit->Text[i]) &&
            NameEdit->Text[i] != ' ' && NameEdit->Text[i] != '-' && NameEdit->Text[i] != '_')
        {
            Application->MessageBoxA("The name of the job contains invalid characters. Please enter a valid name!", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
            NameEdit->SetFocus();
            return;
        }

    if (NameEdit->Text[1] == ' ' || NameEdit->Text[NameEdit->Text.Length()] == ' ')
    {
        Application->MessageBoxA("Job names may not start or end with white space. Please enter a valid name!", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
        NameEdit->SetFocus();
    }
}

void __fastcall TFSKNewForm::LocEditChange(TObject *Sender)
{
    bool IsEmpty;

    IsEmpty = LocEdit->Text.IsEmpty() || NameEdit->Text.IsEmpty();
    if (!IsEmpty && S1_UpdatePath())
    {
        if (S1_VerifySrcPath())
        {
            NextButton->Enabled = true;
            return;
        }
    }
    else
        S1_HidePath();

    NextButton->Enabled = false;
}

void __fastcall TFSKNewForm::SrcEditChange(TObject *Sender)
{
    bool IsEmpty;

    IsEmpty = SrcEdit->Text.IsEmpty() || NameEdit->Text.IsEmpty();
    if (!IsEmpty && S1_VerifySrcPath() && S1_UpdatePath())
        NextButton->Enabled = true;
    else
        NextButton->Enabled = false;
}

void __fastcall TFSKNewForm::LocBrowseButtonClick(TObject *Sender)
{
    char Path[MAX_PATH];

    if (BrowseForFolder(Handle, "Select destination folder.", NULL, NULL, true, Path))
    {
        if (Path[0] == '\\' && Path[1] == '\\')
            Application->MessageBoxA("Jobs cannot be stored on a remote computer!",
                                     FSK_PRODUCT_NAME, MB_ICONEXCLAMATION | MB_OK);
        else
            LocEdit->Text = Path;
    }
}

void __fastcall TFSKNewForm::SrcBrowseButtonClick(TObject *Sender)
{
    char Path[MAX_PATH];

    if (BrowseForFolder(Handle, "Select the drive or folder containing the DVD files.", NULL, NULL, false, Path))
    {
        if (Path[0] == '\\' && Path[1] == '\\')
            Application->MessageBoxA("The DVD must be located on a local drive! Please copy it to your HDD first.",
                                     FSK_PRODUCT_NAME, MB_ICONEXCLAMATION | MB_OK);
        else
            SrcEdit->Text = Path;
    }
}

/*-----------------------------------------------------------------------------
    TFSKNewForm >> Step 1 >> User methods
-----------------------------------------------------------------------------*/

void TFSKNewForm::S1_Begin()
{
    // Disable "Next" button if it's the first time
    if (S1_IsFirstTime)
    {
        NextButton->Enabled = false;
        S1_IsFirstTime = false;

        // Select name edit
        NameEdit->SetFocus();
    }
    else
    {
        NextButton->Enabled = true;

        // Select "Next" button
        NextButton->SetFocus();
    }

    // Hide "Back" button
    BackButton->Visible = false;

    // Set header icon
    HeaderIconList->GetIcon(0, HeaderIcon->Picture->Icon);

    // Set header text
    HeaderTitle->Caption = "Create New Job";
    HeaderComment->Caption = "Enter name, destination folder and DVD source path.";

    // Switch to the page in the notebook
    Notebook->PageIndex = 0;
}

bool TFSKNewForm::S1_End(bool Forward)
{
    // Store DVD source path
    S1_SrcPath = SrcEdit->Text;
    if (S1_SrcPath[S1_SrcPath.Length()] == '\\')
        S1_SrcPath.SetLength(S1_SrcPath.Length() - 1);

    // Check if the destination location is valid
    if (!DirectoryExists(LocEdit->Text))
    {
        Application->MessageBoxA("The destination location is invalid!",
                                 FSK_PRODUCT_NAME, MB_ICONERROR | MB_OK);
        return false;
    }

    // Check if the destination location is on a fixed drive
    char DriveRoot[] = "#:\\\\";

    DriveRoot[0] = JobRoot[1];
    if (GetDriveType(DriveRoot) != DRIVE_FIXED)
    {
        Application->MessageBoxA("Jobs must be stored on an existing fixed drive!",
                                 FSK_PRODUCT_NAME, MB_ICONERROR | MB_OK);

        return false;
    }

    // Check if the job already exists
    if (DirectoryExists(JobRoot))
    {
        Application->MessageBoxA("Job already exists. Change the name or the destination folder!",
                                 FSK_PRODUCT_NAME, MB_ICONERROR | MB_OK);

        return false;
    }

    // Verify if the specified path contains a valid DVD title
    if (!DirectoryExists(S1_SrcPath))
    {
        Application->MessageBoxA("The specified DVD source path does not exist!",
                                 FSK_PRODUCT_NAME, MB_ICONERROR | MB_OK);

        return false;
    }

    if (!S1_VerifyDVD())
    {
        Application->MessageBoxA("Unable to find a valid DVD title at the specified location!",
                                 FSK_PRODUCT_NAME, MB_ICONERROR | MB_OK);

        return false;
    }


    // Store job name
    JobName = NameEdit->Text;

    // Store job path
    JobPath = JobRoot + FSKJ_PATH;

    // Store preview script path
    PreviewPath = JobPath + "preview.avs";

    // Store DVD source path
    if (S1_RootSrcPath)
        DVDPath = S1_SrcPath + "\\VIDEO_TS\\";
    else
        DVDPath = S1_SrcPath + "\\";

    // Store output path
    OutPath = JobRoot;

    // Success
    return true;
}

/*---------------------------------------------------------------------------*/

// Update path label
bool TFSKNewForm::S1_UpdatePath()
{
#define LABEL_TEXT   "Job will be created at %s."
#define LABEL_LIMIT  45               // font type and size dependent!

    int PathLen;

    // Verify if the entered path MAY BE valid
    if (!NameEdit->Text.Pos("\\") && LocEdit->Text.Length() >= 3 &&
        isalpha(LocEdit->Text[1]) && LocEdit->Text[2] == ':' &&
        LocEdit->Text[3] == '\\' && !LocEdit->Text.Pos("\\\\"))
    {
        // Check if the length of the path exceeds the limit
        PathLen = NameEdit->Text.Length() + LocEdit->Text.Length();
        if (LocEdit->Text[LocEdit->Text.Length()] != '\\')
            PathLen++;

        if (PathLen > PATH_LIMIT)
        {
            if (S1_PrevPathLen < PathLen)
                Application->MessageBoxA("The length of the full path for the job exceeds the maximum path length allowed by the system.\nYou must reduce the length of the name or the location.",
                                         FSK_PRODUCT_NAME, MB_ICONEXCLAMATION | MB_OK);
            S1_PrevPathLen = PathLen;

            return false;
        }
        S1_PrevPathLen = PathLen;

        // Update path
        AnsiString Path2;

        if (LocEdit->Text[LocEdit->Text.Length()] == '\\')
            JobRoot = LocEdit->Text + NameEdit->Text;
        else
            JobRoot = LocEdit->Text + "\\" + NameEdit->Text;

        // Shorten path if necessary
        int First = 3;

        Path2 = JobRoot;
        while (Path2.Length() > LABEL_LIMIT)
        {
            int i;

            // Find the second path delimiter
            for (i = First + 1; i < Path2.Length(); i++)
                if (Path2[i+1] == '\\')
                    break;

            if (i < Path2.Length())
            {
                Path2 = Path2.Delete(4, i - 3);
                Path2 = Path2.Insert("...", 4);
                if (First == 3)
                    First = 6;
            }
            else
                break;
        }

        if (Path2.Length() > LABEL_LIMIT)
        {
            int i, Last;

            // The title is too long
            for (i = Path2.Length() - 1; i >= 0; i--)
                if (Path2[i+1] == '\\')
                {
                    Last = i;
                    break;
                }

            Path2 = Path2.Delete((LABEL_LIMIT - Last - 2) / 2 + Last + 1, Path2.Length() - LABEL_LIMIT + 2);
            Path2 = Path2.Insert("...", (LABEL_LIMIT - Last - 2) / 2 + Last + 1);
        }

        // Print path
        AnsiString Label;

        Label.sprintf(LABEL_TEXT, Path2.c_str());
        PathLabel->Caption = Label;

        // Show stuff
        if (!PathIcon->Visible)
        {
            PathIcon->Visible = true;
            PathLabel->Visible = true;
        }

        return true;
    }
    else
        return false;
}

// Hide path icon and label
void TFSKNewForm::S1_HidePath()
{
    if (PathIcon->Visible)
    {
        PathIcon->Visible = false;
        PathLabel->Visible = false;
    }
}

/*---------------------------------------------------------------------------*/

// Verify source path
bool TFSKNewForm::S1_VerifySrcPath()
{
    int PathLen;

    // Verify if the entered path MAY BE valid
    if (SrcEdit->Text.Length() >= 3 && isalpha(SrcEdit->Text[1]) &&
        SrcEdit->Text[2] == ':' && SrcEdit->Text[3] == '\\'
        && !SrcEdit->Text.Pos("\\\\"))
    {
        // Check if the length of the path exceeds the limit
        PathLen = SrcEdit->Text.Length();
        if (SrcEdit->Text[SrcEdit->Text.Length()] != '\\')
            PathLen++;

        if (PathLen > PATH_LIMIT - 10)
        {
            if (S1_PrevSrcPathLen < PathLen)
                Application->MessageBoxA("The length of the full path for the DVD files exceeds the maximum path length allowed by the system.\nYou must move the files to a different location.",
                                         FSK_PRODUCT_NAME, MB_ICONEXCLAMATION | MB_OK);
            S1_PrevSrcPathLen = PathLen;

            return false;
        }
        S1_PrevSrcPathLen = PathLen;
        return true;
    }
    else
        return false;
}

// Verify if S1_SrcPath points to a valid VIDEO_TS folder
bool TFSKNewForm::S1_VerifyDVD(AnsiString Path)
{
    AnsiString IfoName, VobName;
    TSearchRec SearchRec;
    int ID;

    // Find the first VTS IFO file
    if (FindFirst(Path + "\\VTS_??_0.IFO", faAnyFile, SearchRec) || !SearchRec.Size)
        return false;

    // Check if the corresponding VOB file exists
    IfoName = SearchRec.Name.UpperCase();
    sscanf(IfoName.c_str(), "VTS_%d_0.IFO", &ID);
    VobName.sprintf("VTS_%.2d_1.VOB", ID);
    if (FileExists(Path + "\\" + VobName))
    {
        FindClose(SearchRec);
        return true;
    }
    else
    {
        // Keep searching for a valid VTS
        while (!FindNext(SearchRec))
        {
            IfoName = SearchRec.Name.UpperCase();
            sscanf(IfoName.c_str(), "VTS_%d_0.IFO", &ID);
            VobName.sprintf("VTS_%.2d_1.VOB", ID);
            if (FileExists(Path + "\\" + VobName))
            {
                FindClose(SearchRec);
                return true;
            }
        }

        FindClose(SearchRec);
        return false;
    }
}

// Verify if S1_SrcPath points to a valid DVD title
bool TFSKNewForm::S1_VerifyDVD()
{
    AnsiString RootPath, VTSPath;

    // Get DVD path
    RootPath = S1_SrcPath;
    VTSPath = RootPath + "\\VIDEO_TS";

    // Check for DVD files
    if (S1_VerifyDVD(VTSPath))
    {
        S1_RootSrcPath = true;
        return true;
    }
    else
    {
        if (S1_VerifyDVD(RootPath))
        {
            S1_RootSrcPath = false;
            return true;
        }
        else
            return false;
    }
}

/*=============================================================================
    TFSKNewForm >> Step 2
=============================================================================*/

/*-----------------------------------------------------------------------------
    TFSKNewForm >> Step 2 >> User methods
-----------------------------------------------------------------------------*/

void TFSKNewForm::S2_Begin()
{
    // Enable "Next" and "Back" button
    NextButton->Enabled = true;
    BackButton->Enabled = true;
    BackButton->Visible = true;

    // Set header icon
    HeaderIconList->GetIcon(1, HeaderIcon->Picture->Icon);

    // Set header text
    HeaderTitle->Caption   = "DVD Analysis";
    HeaderComment->Caption = "Please wait while analyzing the DVD title...";

    // Disable "Next" button
    NextButton->Enabled = false;

    // Clear log
    LogList->Clear();

    // Zero progress bar position
    ProgressBar->Position = 0;

    // Hide time
    TimeTextLabel->Visible = false;
    TimeLabel->Visible = false;

    // Switch to the page in the notebook
    Notebook->PageIndex = 1;

    // Reset stuff when entering the preview and config pass
    S3_Reset   = true;
    S4_Reset   = true;
    S3_PrevVTS = S3_PrevPGC = S3_PrevAngle = -1;

    // Start the analyzation
    S2_AbortFlag = S2_Finished = false;
    for (int i = 0; i < 100; i++)
        TitleInfo[i].PGCNum = 0;

    // Update log
    AddLogEvent(LSI_Info, "Analysis started");

    // Start the thread
    //if (AnalyzeThread)
    //    delete AnalyzeThread;
    AnalyzeThread = new TFSKAnalyzeThread(false);
    S2_Active = true;
}

bool TFSKNewForm::S2_End(bool Forward)
{
    // Terminate thread if active
    if (S2_Active)
    {
        if (!S2_Abort(true))
            return false;
    }

    // Remove job directory if going backwards
    if (S2_Finished && !Forward && DirectoryExists(JobRoot))
    {
        if (Application->MessageBoxA(MSG_DISCARD_ANALYSIS, FSK_PRODUCT_NAME, MB_YESNO | MB_ICONEXCLAMATION) == mrYes)
            RemoveDirTree(JobRoot);
        else
            return false;
    }

    return true;
}

/*---------------------------------------------------------------------------*/

// Analysis finished
void TFSKNewForm::S2_Finish(bool Success)
{
    // Process aborted, S2_Abort will do the job
    if (S2_AbortFlag)
        return;

    if (Success)
    {
        // Hide time label
        TimeTextLabel->Visible = false;
        TimeLabel->Visible = false;

        // Enable "Next" button
        NextButton->Enabled = true;

        // The thread is terminated
        S2_Active = false;

        // Set flags
        S2_Finished = true;

        // Set progress bar position to 100%
        ProgressBar->Position = 100;

        // Update log
        AddLogEvent(LSI_Info, "Analysis finished");

        // Change header comment text
        HeaderComment->Caption = "Press the \"Next\" button to continue!";

        // Beep
        MessageBeep(MB_OK);

        // Select "Next" button
        NextButton->SetFocus();
    }
    else
    {
        // Set flags
        S2_Finished = false;

        // The thread is terminated
        S2_Active = false;

        // Close window and open menu
        Release();
        FSKMenuForm->Enabled = true;
    }
}

// Abort analysis
bool TFSKNewForm::S2_Abort(bool Back)
{
    int Result;

    // Suspend the analysis thread
    AnalyzeThread->SuspendQuery();

    if (Back)
        Result = Application->MessageBoxA("Do you really want abort the analysis?", FSK_PRODUCT_NAME, MB_YESNO | MB_ICONEXCLAMATION);
    else
        Result = Application->MessageBoxA(MSG_CLOSE_WIZARD, FSK_PRODUCT_NAME, MB_YESNO | MB_ICONEXCLAMATION);

    if (Result == mrNo)
    {
        // Resume thread
        AnalyzeThread->ResumeQuery();
        return false;
    }
    else
    {
        S2_AbortFlag = true;

        // Terminate thread
        AnalyzeThread->ResumeQuery();
        AnalyzeThread->Terminate();
        AnalyzeThread->WaitFor();

        S2_Active = false;

        // Delete job directory
        if (DirectoryExists(JobRoot))
            RemoveDirTree(JobRoot);

        // Update log and show a message box
        AddLogEvent(LSI_Error, "Analysis aborted");
        Application->MessageBoxA("Analysis aborted by user.", FSK_PRODUCT_NAME, MB_ICONERROR | MB_OK);

        return true;
    }
}

/*---------------------------------------------------------------------------*/

// Add a new event to the log list
void TFSKNewForm::AddLogEvent(ELogStateIcon StateIcon, AnsiString Event)
{
    TListItem *Item;

    Item = LogList->Items->Add();
    Item->StateIndex = StateIcon;
    Item->SubItems->Add(TimeToStr(Now()));
    Item->SubItems->Add(Event);
    LogList->Scroll(0, LogList->Items->Count * 20);
}

/*=============================================================================
    TFSKNewForm >> Step 3
=============================================================================*/

/*-----------------------------------------------------------------------------
    TFSKNewForm >> Step 3 >> Events
-----------------------------------------------------------------------------*/

void __fastcall TFSKNewForm::TitleTreeChange(TObject *Sender, TTreeNode *Node)
{
    AnsiString Text;
    AnsiString IndexName, IfoPath, OutName;     // paths
    bool       Result;                          // return value of a function
    bool       _S3_TitleTreeRebuildOnly;

    _S3_TitleTreeRebuildOnly = S3_TitleTreeRebuildOnly;
    S3_TitleTreeRebuildOnly  = false;

    // Disable stuff
    NextButton->Enabled = false;
    PreviewBar->Enabled = false;

    if (!_S3_TitleTreeRebuildOnly)
    {
        TitleLabel->Caption   = TXT_NO_TITLE;
        PreviewImage->Visible = false;

        // Show "No Title" image
        NoTitleImage->Visible = true;

        // Close previous title
        if (S3_Selected)
        {
            m2PreviewClose();
            S3_Selected = false;
        }

        // Set the trackbar to the middle
        PreviewBar->Position = PreviewBar->Max / 2;

        // Check if the selection is valid
        if (!((Node->StateIndex == 3) || ((Node->StateIndex == 2) && !Node->Count)))
            return;

        // Refresh form and show hourglass
        Refresh();
        Screen->Cursor = crHourGlass;

        // Get selection data
        if (Node->StateIndex == 3)
        {
            sscanf(Node->Text.c_str(), "Angle %d", &Angle);
            sscanf(Node->Parent->Text.c_str(), "PGC %d", &PGC);
            sscanf(Node->Parent->Parent->Text.c_str(), "VTS_%d", &VTS);

            Text.sprintf(" VTS_%.2d > PGC %d > Angle %d", VTS, PGC, Angle);
        }
        else
        {
            Angle = 0;
            sscanf(Node->Text.c_str(), "PGC %d", &PGC);
            sscanf(Node->Parent->Text.c_str(), "VTS_%d", &VTS);

            Text.sprintf(" VTS_%.2d > PGC %d", VTS, PGC);
        }

        // Set title info
        TitleLabel->Caption = Text;
    }

    // Rebuild video index
    IndexName = JobPath;
    IndexName.cat_printf("vts%.2d", VTS);
    IfoPath = DVDPath;
    IfoPath.cat_printf("VTS_%.2d_0.IFO", VTS);
    OutName = JobPath;
    OutName.cat_printf("vts%.2d_pgc%.2d_a%d", VTS, PGC, Angle);

    S3_FileName = OutName;
    S3_FileName.cat_printf(".m2vi");

    // Try to open index
    Refresh();

    if (!FileExists(S3_FileName) || !m2PreviewOpen(S3_FileName.c_str(), S3_SelWidth, S3_SelHeight, _NumFrames, int(FSKNewForm->TitleInfo[VTS].Video.FilmPurity) > 0))
    {
        // Rebuild index
        Result = m2Rebuild(IndexName.c_str(), IfoPath.c_str(), OutName.c_str(), true, false, false, PGC, Angle);

        // Check if the rebuild operation was successful and open index
        if (!Result || !m2PreviewOpen(S3_FileName.c_str(), S3_SelWidth, S3_SelHeight, _NumFrames, int(FSKNewForm->TitleInfo[VTS].Video.FilmPurity) > 0))
        {
PreviewError:
            Screen->Cursor = crDefault;
            Application->MessageBoxA(MSG_PREVIEW_ERROR, FSK_PRODUCT_NAME, MB_ICONERROR | MB_OK);
            return;
        }
    }

    // Resize preview image
    if (!_S3_TitleTreeRebuildOnly)
    {
        if (TitleInfo[VTS].Video.IsWide)
        {
            // 16:9
            PreviewImage->Top    = S3_Pre169Top;
            PreviewImage->Height = S3_Pre169Height;
        }
        else
        {
            // 4:3
            PreviewImage->Top    = S3_Pre43Top;
            PreviewImage->Height = S3_Pre43Height;
        }

        // Init frame
        S3_PreviewFrame->Width = S3_SelWidth / 2;
        S3_PreviewFrame->Height = S3_SelHeight / 2;
    }

    Refresh();

    // Set mouse pointer to default
    Screen->Cursor = crDefault;

    // Show selection
    PreviewBar->Enabled = true;

    // Ready for preview
    S3_Selected = true;

    // Show frame
    S3_PreviewActive = false;

    if (!_S3_TitleTreeRebuildOnly)
    {
        PreviewBarChange(Sender);

        NoTitleImage->Visible = false;
        PreviewImage->Visible = true;
    }

    NextButton->Enabled = _NumFrames > 1;
}

void __fastcall TFSKNewForm::TitleTreeKeyPress(TObject *Sender, char &Key)
{
    // If "Enter" is pressed, click the "Next" button
    if (Key == 13 && NextButton->Enabled)
    {
        Key = 0;
        NextButtonClick(Sender);
    }
}

void __fastcall TFSKNewForm::PreviewBarChange(TObject *Sender)
{
    if (!S3_Selected)
        return;

    int            i, j;         // counters
    int            Frame;        // frame index
    unsigned char *Src, *Dest;   // source and destination pixel pointers

    if (S3_PreviewActive)
        return;

    S3_PreviewActive = true;

    // Get frame index
    Frame = ((_NumFrames - 1) * PreviewBar->Position) / PreviewBar->Max;

    // Get frame data
    if (!(Src = m2PreviewGet(Frame)))
    {
        S3_Selected = false;

        // Show error box
        Application->MessageBoxA("Error detected in the selected title! The DVD is probably damaged!", FSK_PRODUCT_NAME, MB_ICONERROR | MB_OK);

        // Disable stuff
        NextButton->Enabled   = false;
        PreviewBar->Enabled   = false;
        TitleLabel->Caption   = TXT_NO_TITLE;
        PreviewImage->Visible = false;

        // Show "No Title" image
        NoTitleImage->Visible = true;

        // Close title
        m2PreviewClose();

        // Set the trackbar to the middle
        PreviewBar->Position = PreviewBar->Max / 2;

        return;
    }

    // Copy pixels to frame bitmap
    for (i = 0; i < S3_SelHeight / 2; i++)
    {
        Dest = (unsigned char *)S3_PreviewFrame->ScanLine[i];

        for (j = 0; j < S3_SelWidth / 2; j++)
        {
            *Dest++ = *Src++;
            *Dest++ = *Src++;
            *Dest++ = *Src++;

            Src += 3;
        }

        Src += S3_SelWidth * 3;
    }

    // Show frame bitmap
    PreviewImage->Picture->Bitmap = S3_PreviewFrame;
    PreviewImage->Refresh();

    S3_PreviewActive = false;
}

/*-----------------------------------------------------------------------------
    TFSKNewForm >> Step 3 >> User methods
-----------------------------------------------------------------------------*/

void TFSKNewForm::S3_Begin()
{
    int        i, j, k;                        // counters
    TTreeNode *VTSNode, *PGCNode, *AngleNode;  // tree node pointers
    AnsiString NodeText;                       // text of a node
    TTreeNode *LongNode;                       // longest PGC node pointer
    int        LongTimeSec, LongTimeFrm;       // longest PGC time in seconds + frames
    int        TimeSec, TimeFrm;               // current PGC time in seconds + frames

    S3_IsFirstEndCall = true;
    S3_PreviewActive  = false;
    S3_TitleTreeRebuildOnly = false;

    // Set header icon
    HeaderIconList->GetIcon(2, HeaderIcon->Picture->Icon);

    // Set header text
    HeaderTitle->Caption = "Select title";
    HeaderComment->Caption = "Select the PGC/angle you want to transcode.";

    // Check if we have to reset
    if (!S3_Reset)
    {
        // Switch to the page in the notebook
        Notebook->PageIndex = 2;

        // Open preview index
        NextButton->Enabled = false;
        Screen->Cursor      = crHourGlass;
        S3_Selected         = false;

        // Refresh form
        Refresh();

        if (m2PreviewOpen(S3_FileName.c_str(), S3_SelWidth, S3_SelHeight, _NumFrames, int(FSKNewForm->TitleInfo[VTS].Video.FilmPurity) > 0))
        {
            // Success
            Screen->Cursor      = crDefault;
            S3_Selected         = true;
            NextButton->Enabled = true;
        }
        else
        {
            // Try to rebuild index again
            S3_TitleTreeRebuildOnly = true;
            TitleTreeChange(NULL, TitleTree->Selected);
        }

        // Select title tree
        TitleTree->SetFocus();

        return;
    }

    // Disable "Next" button
    NextButton->Enabled = false;

    // Initialization
    S3_Selected = false;
    LongTimeSec = LongTimeFrm = 0;

    // Show "No Title" image
    PreviewImage->Top     = S3_Pre43Top;
    PreviewImage->Height  = S3_Pre43Height;
    NoTitleImage->Visible = true;

    // Set preview bar to the middle
    PreviewBar->Position = PreviewBar->Max / 2;

    // Clear title tree
    TitleTree->Items->Clear();

    // Add items
    for (i = 1; i < 100; i++)
    {
        if (TitleInfo[i].PGCNum)
        {
            // Add VTS
            NodeText.sprintf("VTS_%.2d", i);
            VTSNode = TitleTree->Items->Add(NULL, NodeText);
            VTSNode->StateIndex = 1;

            // Add PGCs
            for (j = 1; j <= TitleInfo[i].PGCNum; j++)
            {
                NodeText.sprintf("PGC %d [%.2d:%.2d:%.2d]", j, TitleInfo[i].PGC[j].Length.hour,
                                                               TitleInfo[i].PGC[j].Length.minute,
                                                               TitleInfo[i].PGC[j].Length.second);

                PGCNode = TitleTree->Items->AddChild(VTSNode, NodeText);
                PGCNode->StateIndex = 2;

                // Add angles
                if (TitleInfo[i].PGC[j].AngleNum > 1)
                {
                    for (k = 1; k <= TitleInfo[i].PGC[j].AngleNum; k++)
                    {
                        NodeText.sprintf("Angle %d", k);
                        AngleNode = TitleTree->Items->AddChild(PGCNode, NodeText);
                        AngleNode->StateIndex = 3;
                    }
                }

                // Check if it's the longest PGC
                TimeSec = TitleInfo[i].PGC[j].Length.hour * 3600 + TitleInfo[i].PGC[j].Length.minute * 60 + TitleInfo[i].PGC[j].Length.second;
                TimeFrm = TitleInfo[i].PGC[j].Length.frame_u & DVD_TIME_MASK;

                if ((TimeSec > LongTimeSec) || ((TimeSec == LongTimeSec) && (TimeFrm > LongTimeFrm)))
                {
                    LongTimeSec = TimeSec;
                    LongTimeFrm = TimeFrm;
                    LongNode = PGCNode;
                }
            }
        }
    }

    // Expand tree
    TitleTree->FullExpand();

    // Next time we don't have to reset
    S3_Reset = false;

    // Switch to the page in the notebook
    Notebook->PageIndex = 2;

    // Select longest PGC/angle
    if (LongNode->Count)
        LongNode = LongNode->getFirstChild();

    TShiftState ShiftState;
    ShiftState << ssLeft;
    TitleTree->Select(LongNode, ShiftState);

    // Select title tree
    TitleTree->SetFocus();
}

bool TFSKNewForm::S3_End(bool Forward)
{
    if (Forward)
    {
        if (!S3_IsFirstEndCall)
            return true;
        S3_IsFirstEndCall = false;

        // Close files
        if (S3_Selected)
        {
            m2PreviewClose();
            S3_Selected = false;
        }

        // Check if the selection is changed
        if ((VTS == S3_PrevVTS) && (PGC == S3_PrevPGC) && (Angle == S3_PrevAngle))
        {
            // We don't have to reset encoding params
            S4_Reset = false;
        }
        else
        {
            // We have to reset encoding params
            S4_Reset = true;

            // Check number of frames
            if (TitleInfo[VTS].PGC[PGC].Video.NumFrames < 0)
            {
                // Show wait form, rebuild index with audio and crop borders
                TThread *Thread;

                Thread = new TFSKSelectThread();
                FSKNewWaitForm = new TFSKNewWaitForm(this, Thread);
                FSKNewWaitForm->Show();
                return false;
            }
            else
            {
                // Adjust rebuild index filename
                ReIndexName.printf("vts%.2d_pgc%.2d_a%d", VTS, PGC, Angle);
            }
        }
    }
    else
    {
        if (Application->MessageBoxA(MSG_DISCARD_ANALYSIS, FSK_PRODUCT_NAME, MB_YESNO | MB_ICONEXCLAMATION) == mrYes)
        {
            // Close files
            if (S3_Selected)
            {
                m2PreviewClose();
                S3_Selected = false;
            }

            // Remove job directory
            RemoveDirTree(JobRoot);
        }
        else
            return false;
    }

    return true;
}

/*=============================================================================
    TFSKNewForm >> Step 4
=============================================================================*/

/*-----------------------------------------------------------------------------
    TFSKNewForm >> Step 4 >> Events
-----------------------------------------------------------------------------*/

void __fastcall TFSKNewForm::EncModeFixedRadioClick(TObject *Sender)
{
    if (!S4_IsActive)
        return;

    // Enable self
    VolCheck->Checked = true;
    VolCheckClick(Sender);
    VolCheck->Enabled = false;

    // Disable bitrate and quality controls
    EncModeBitrateEdit->Enabled   = false;
    EncModeBitrateUpDown->Enabled = false;
    EncModeBitrateLabel->Enabled  = false;

    EncModeQualityEdit->Enabled   = false;
    EncModeQualityUpDown->Enabled = false;
    EncModeQualityLabel->Enabled  = false;

    // Enable volume count controls
    if (VolCheck->Checked)
    {
        VolNumLabel->Enabled  = true;
        VolNumEdit->Enabled   = true;
        VolNumUpDown->Enabled = true;
    }

    // Enable auto resize & audio bitrate
    VideoResAutoCheck->Enabled = true;
    Audio1BitrateAutoRadio->Enabled = true;

    // Adjust bitrate and output size if necessary
    if (IsAudio)
    {
        EncModeBitrateUpDown->Min = AudioPresets[Audio1BitrateTrack->Position].Bitrate + FSK_MIN_VIDEO_BITRATE + 2;
        S4_MinOutputSize = int(ceil(double(AudioPresets[Audio1BitrateTrack->Position].Bitrate + FSK_MIN_VIDEO_BITRATE + 2) * OriNumSeconds / (1024 * 8)));
        if (!S4_MinOutputSize)
            S4_MinOutputSize = 1;
        VolUpDown->Min = int(ceil(double(S4_MinOutputSize) / VolNumUpDown->Position));
    }
    else
    {
        EncModeBitrateUpDown->Min = FSK_MIN_VIDEO_BITRATE;
        S4_MinOutputSize = int(ceil(double(FSK_MIN_VIDEO_BITRATE) * OriNumSeconds / (1024 * 8)));
        if (!S4_MinOutputSize)
            S4_MinOutputSize = 1;
        VolUpDown->Min = int(ceil(double(S4_MinOutputSize) / VolNumUpDown->Position));
    }

    EncModeBitrateEdit->Text = IntToStr(EncModeBitrateUpDown->Position);
    VolEdit->Text = IntToStr(VolUpDown->Position);
    VolNumEdit->Text = IntToStr(VolNumUpDown->Position);

    // Update video complexity
    S4_UpdateVideoComplexity(FSKVideoComplexityBitrate);

    // Update bitrate
    S4_UpdateBitrate();
}

void __fastcall TFSKNewForm::EncModeBitrateRadioClick(TObject *Sender)
{
    if (!S4_IsActive)
        return;

    // Enable self
    EncModeBitrateEdit->Enabled   = true;
    EncModeBitrateUpDown->Enabled = true;
    EncModeBitrateLabel->Enabled  = true;

    VolCheck->Enabled = true;

    // Disable quality controls
    EncModeQualityEdit->Enabled   = false;
    EncModeQualityUpDown->Enabled = false;
    EncModeQualityLabel->Enabled  = false;

    // Disable volume count controls
    VolNumLabel->Enabled  = true;
    VolNumEdit->Enabled   = false;
    VolNumUpDown->Enabled = false;

    // Enable auto resize & audio bitrate
    VideoResAutoCheck->Enabled = true;
    Audio1BitrateAutoRadio->Enabled = true;

    // Adjust bitrate and output size if necessary
    if (IsAudio)
        EncModeBitrateUpDown->Min = AudioPresets[Audio1BitrateTrack->Position].Bitrate + FSK_MIN_VIDEO_BITRATE + 2;
    else
        EncModeBitrateUpDown->Min = FSK_MIN_VIDEO_BITRATE;

    EncModeBitrateEdit->Text = IntToStr(EncModeBitrateUpDown->Position);

    // Update video complexity
    S4_UpdateVideoComplexity(FSKVideoComplexityBitrate);

    // Update bitrate
    S4_UpdateBitrate();
}

void __fastcall TFSKNewForm::EncModeQualityRadioClick(TObject *Sender)
{
    if (!S4_IsActive)
        return;

    // Enable self
    EncModeQualityEdit->Enabled   = true;
    EncModeQualityUpDown->Enabled = true;
    EncModeQualityLabel->Enabled  = true;

    VolCheck->Enabled = true;

    // Disable bitrate controls
    EncModeBitrateEdit->Enabled   = false;
    EncModeBitrateUpDown->Enabled = false;
    EncModeBitrateLabel->Enabled  = false;

    // Disable volume count controls
    VolNumLabel->Enabled  = false;
    VolNumEdit->Enabled   = false;
    VolNumUpDown->Enabled = false;

    // Disable auto resize & audio bitrate
    VideoResManuCheck->Checked  = true;
    VideoResAutoCheck->Enabled = false;

    Audio1BitrateManuRadio->Checked = true;
    Audio1BitrateAutoRadio->Enabled = false;

    // Update video complexity
    S4_UpdateVideoComplexity(FSKVideoComplexityQuality);

    // Update bitrate
    S4_UpdateBitrate();
}

void __fastcall TFSKNewForm::VolCheckClick(TObject *Sender)
{
    if (!S4_IsActive)
        return;

    // Enable/disable self
    if (VolCheck->Checked)
    {
        VolCombo->Enabled     = true;

        if (VolCombo->ItemIndex)
        {
            VolEdit->Enabled    = false;
            VolUpDown->Enabled  = false;
        }
        else
        {
            VolEdit->Enabled    = true;
            VolUpDown->Enabled  = true;
        }

        VolLabel->Enabled     = true;

        if (EncModeFixedRadio->Checked)
        {
            VolNumLabel->Enabled  = true;
            VolNumEdit->Enabled   = true;
            VolNumUpDown->Enabled = true;
        }
    }
    else
    {
        VolCombo->Enabled     = false;
        VolEdit->Enabled      = false;
        VolUpDown->Enabled    = false;
        VolLabel->Enabled     = false;
        VolNumLabel->Enabled  = false;
        VolNumEdit->Enabled   = false;
        VolNumUpDown->Enabled = false;
    }
}

void __fastcall TFSKNewForm::NumEditKeyPress(TObject *Sender, char &Key)
{
    // Ignore nondigit characters
    if ((Key != 8) /* Backspace */ && !isdigit(Key))
    {
        Key = 0; // No key pressed
        MessageBeep(MB_OK);
    }
}

void __fastcall TFSKNewForm::EncModeBitrateEditChange(TObject *Sender)
{
    if (!S4_IsActive)
        return;

    // Update bitrate
    if (!EncModeBitrateEdit->Text.IsEmpty())
    {
        try
        {
            if (EncModeBitrateEdit->Text.ToInt() >= EncModeBitrateUpDown->Min)
                S4_UpdateBitrate();
        }
        catch (...)
        {
        }
    }
}

void __fastcall TFSKNewForm::EncModeQualityEditChange(TObject *Sender)
{
    if (!S4_IsActive)
        return;

    try
    {
        EncModeQualityEdit->Text.ToInt();
    }
    catch (...)
    {
        return;
    }

    // Update bitrate
    S4_UpdateBitrate();
}


void __fastcall TFSKNewForm::VolEditChange(TObject *Sender)
{
    if (!S4_IsActive)
        return;

    // Update bitrate
    if (!VolEdit->Text.IsEmpty())
    {
        try
        {
            VolEdit->Text.ToInt();
        }
        catch (...)
        {
            return;
        }

        S4_UpdateBitrate();
    }
}

void __fastcall TFSKNewForm::VolNumEditChange(TObject *Sender)
{
    if (!S4_IsActive)
        return;

    // Update minimum volume size
    VolUpDown->Min = int(ceil(double(S4_MinOutputSize) / VolNumUpDown->Position));
    VolEdit->Text  = IntToStr(VolUpDown->Position);

    // Update bitrate
    S4_UpdateBitrate();
}

void __fastcall TFSKNewForm::EncModeBitrateEditExit(TObject *Sender)
{
    if (!EncModeBitrateEdit->Text.IsEmpty())
    {
        try
        {
            if (EncModeBitrateEdit->Text.ToInt())
                EncModeBitrateEdit->Text = IntToStr(EncModeBitrateUpDown->Position);
        }
        catch (...)
        {
            Application->MessageBoxA("Please enter a valid integer greater than 0.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
            EncModeBitrateEdit->SetFocus();
        }
    }
    else
    {
        Application->MessageBoxA("Please enter an integer greater than 0.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
        EncModeBitrateEdit->SetFocus();
    }
}

void __fastcall TFSKNewForm::EncModeQualityEditExit(TObject *Sender)
{
    if (!EncModeQualityEdit->Text.IsEmpty())
    {
        try
        {
            EncModeQualityEdit->Text.ToInt();
        }
        catch (...)
        {
            Application->MessageBoxA("Please enter a valid integer between 0 and 100.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
            EncModeQualityEdit->SetFocus();
            return;
        }

        EncModeQualityEdit->Text = IntToStr(EncModeQualityUpDown->Position);
    }
    else
    {
        Application->MessageBoxA("Please enter an integer between 0 and 100.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
        EncModeQualityEdit->SetFocus();
    }
}

void __fastcall TFSKNewForm::VolEditExit(TObject *Sender)
{
    if (!VolEdit->Text.IsEmpty())
    {
        try
        {
            if (VolEdit->Text.ToInt())
                VolEdit->Text = IntToStr(VolUpDown->Position);
        }
        catch (...)
        {
           Application->MessageBoxA("Please enter a valid integer greater than 0.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
           VolEdit->SetFocus();
        }
    }
    else
    {
        Application->MessageBoxA("Please enter an integer greater than 0.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
        VolEdit->SetFocus();
    }
}

void __fastcall TFSKNewForm::VolComboChange(TObject *Sender)
{
    if (!S4_IsActive)
        return;

    if (VolCombo->ItemIndex)
    {
        VolEdit->Enabled   = false;
        VolUpDown->Enabled = false;

        VolEdit->Text = IntToStr(FSKVolumeSize[VolCombo->ItemIndex - 1].Size);
        Refresh();
    }
    else
    {
        VolEdit->Enabled   = true;
        VolUpDown->Enabled = true;
    }

    // Update bitrate
    S4_UpdateBitrate();
}

/*---------------------------------------------------------------------------*/

void __fastcall TFSKNewForm::VideoResEditKeyPress(TObject *Sender, char &Key)
{
    // Ignore nondigit characters
    if (!isdigit(Key) && (Key != 8) /* Backspace */)
    {
        //Key = 27; /* Escape */
        Key = 0; // No key pressed
        MessageBeep(MB_OK);
    }
}

void __fastcall TFSKNewForm::VideoResWEditChange(TObject *Sender)
{
    int Width, Height;

    if (!S4_IsActive)
        return;

    if (VideoResWEdit->Text.IsEmpty() || VideoResHEdit->Text.IsEmpty())
        return;
    else
    {
        try
        {
            VideoResWEdit->Text.ToInt();
            VideoResHEdit->Text.ToInt();
        }
        catch (...)
        {
            return;
        }
    }

    S4_IsActive = false;

    // Calculate height
    Width = VideoResWUpDown->Position;
    if (Width)
        Height = int(ceil((double)Width / VideoAR));
    else
        Height = 0;

    // Update resolution
    if (VideoResWEdit->Focused())
        VideoResHEdit->Text = IntToStr(Height);
    else
    {
        // Adjust resolution
        S4_AdjustResolution(Width, Height, VideoAR, true);

        VideoResWEdit->Text = IntToStr(Width);
        VideoResHEdit->Text = IntToStr(Height);
    }

    S4_IsActive = true;
}

void __fastcall TFSKNewForm::VideoResHEditChange(TObject *Sender)
{
    int Width, Height;

    if (!S4_IsActive)
        return;

    if (VideoResWEdit->Text.IsEmpty() || VideoResHEdit->Text.IsEmpty())
        return;
    else
    {
        try
        {
            VideoResWEdit->Text.ToInt();
            VideoResHEdit->Text.ToInt();
        }
        catch (...)
        {
            return;
        }
    }

    S4_IsActive = false;

    // Calculate width
    Height = VideoResHUpDown->Position;
    if (Height)
        Width = int(floor((double)Height * VideoAR));
    else
        Width = 0;

    // Update resolution
    if (VideoResHEdit->Focused())
        VideoResWEdit->Text = IntToStr(Width);
    else
    {
        // Adjust resolution
        S4_AdjustResolution(Width, Height, VideoAR, false);

        VideoResWEdit->Text = IntToStr(Width);
        VideoResHEdit->Text = IntToStr(Height);
    }

    S4_IsActive = true;
}

void __fastcall TFSKNewForm::VideoResWEditExit(TObject *Sender)
{
    int Width, Height;

    if (VideoResWEdit->Text.IsEmpty())
    {
Error:
        Application->MessageBoxA("Please enter an integer greater than 0.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
        VideoResWEdit->SetFocus();
        return;
    }
    else
    {
        try
        {
            if (!VideoResWEdit->Text.ToInt())
                goto Error;
        }
        catch (...)
        {
            Application->MessageBoxA("Please enter a valid integer greater than 0.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
            VideoResWEdit->SetFocus();
            return;
        }
    }

    S4_IsActive = false;

    // Get width
    Width = VideoResWUpDown->Position;

    // Adjust resolution
    S4_AdjustResolution(Width, Height, VideoAR, true);

    // Replace resolution
    VideoResWEdit->Text = IntToStr(Width);
    VideoResHEdit->Text = IntToStr(Height);

    S4_IsActive = true;
}

void __fastcall TFSKNewForm::VideoResHEditExit(TObject *Sender)
{
    int Width, Height;

    if (VideoResHEdit->Text.IsEmpty())
    {
Error:
        Application->MessageBoxA("Please enter an integer greater than 0.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
        VideoResHEdit->SetFocus();
        return;
    }
    else
    {
        try
        {
             if (!VideoResHEdit->Text.ToInt())
                goto Error;
        }
        catch (...)
        {
            Application->MessageBoxA("Please enter a valid integer greater than 0.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
            VideoResHEdit->SetFocus();
            return;
        }
    }

    S4_IsActive = false;

    // Get width
    Width = VideoResWUpDown->Position;

    // Adjust resolution
    S4_AdjustResolution(Width, Height, VideoAR, true);

    // Replace resolution
    VideoResWEdit->Text = IntToStr(Width);
    VideoResHEdit->Text = IntToStr(Height);

    S4_IsActive = true;
}

void __fastcall TFSKNewForm::VideoResAutoCheckClick(TObject *Sender)
{
    if (!S4_IsActive)
        return;

    VideoResWEdit->Enabled   = false;
    VideoResWUpDown->Enabled = false;
    VideoResHEdit->Enabled   = false;
    VideoResHUpDown->Enabled = false;

    // Update bitrate
    S4_UpdateBitrate();
}

void __fastcall TFSKNewForm::VideoResManuCheckClick(TObject *Sender)
{
    if (!S4_IsActive)
        return;

    VideoResWEdit->Enabled   = true;
    VideoResWUpDown->Enabled = true;
    VideoResHEdit->Enabled   = true;
    VideoResHUpDown->Enabled = true;

    // Update bitrate
    S4_UpdateBitrate();
}

void __fastcall TFSKNewForm::VideoCropNoCheckClick(TObject *Sender)
{
    if (!S4_IsActive)
        return;

    VideoCropTEdit->Enabled   = false;
    VideoCropTUpDown->Enabled = false;
    VideoCropBEdit->Enabled   = false;
    VideoCropBUpDown->Enabled = false;
    VideoCropLEdit->Enabled   = false;
    VideoCropLUpDown->Enabled = false;
    VideoCropREdit->Enabled   = false;
    VideoCropRUpDown->Enabled = false;
    VideoCropAdjButton->Enabled = false;

    VideoCropTEdit->Text = "0";
    VideoCropBEdit->Text = "0";
    VideoCropLEdit->Text = "0";
    VideoCropREdit->Text = "0";
}

void __fastcall TFSKNewForm::VideoCropAutoCheckClick(TObject *Sender)
{
    if (!S4_IsActive)
        return;

    VideoCropTEdit->Enabled   = false;
    VideoCropTUpDown->Enabled = false;
    VideoCropBEdit->Enabled   = false;
    VideoCropBUpDown->Enabled = false;
    VideoCropLEdit->Enabled   = false;
    VideoCropLUpDown->Enabled = false;
    VideoCropREdit->Enabled   = false;
    VideoCropRUpDown->Enabled = false;
    VideoCropAdjButton->Enabled = false;

    VideoCropTUpDown->Max = TitleInfo[VTS].PGC[PGC].Video.Top;
    VideoCropBUpDown->Max = TitleInfo[VTS].Video.Height - TitleInfo[VTS].PGC[PGC].Video.Height - TitleInfo[VTS].PGC[PGC].Video.Top;
    VideoCropLUpDown->Max = TitleInfo[VTS].PGC[PGC].Video.Left;
    VideoCropRUpDown->Max = TitleInfo[VTS].Video.Width - TitleInfo[VTS].PGC[PGC].Video.Width - TitleInfo[VTS].PGC[PGC].Video.Left;

    VideoCropTEdit->Text = IntToStr(TitleInfo[VTS].PGC[PGC].Video.Top);
    VideoCropBEdit->Text = IntToStr(TitleInfo[VTS].Video.Height - TitleInfo[VTS].PGC[PGC].Video.Height - TitleInfo[VTS].PGC[PGC].Video.Top);
    VideoCropLEdit->Text = IntToStr(TitleInfo[VTS].PGC[PGC].Video.Left);
    VideoCropREdit->Text = IntToStr(TitleInfo[VTS].Video.Width - TitleInfo[VTS].PGC[PGC].Video.Width - TitleInfo[VTS].PGC[PGC].Video.Left);
}

void __fastcall TFSKNewForm::VideoCropManuCheckClick(TObject *Sender)
{
    if (!S4_IsActive)
        return;

    VideoCropTEdit->Enabled   = true;
    VideoCropTUpDown->Enabled = true;
    VideoCropBEdit->Enabled   = true;
    VideoCropBUpDown->Enabled = true;
    VideoCropLEdit->Enabled   = true;
    VideoCropLUpDown->Enabled = true;
    VideoCropREdit->Enabled   = true;
    VideoCropRUpDown->Enabled = true;
    VideoCropAdjButton->Enabled = true;
}

void __fastcall TFSKNewForm::VideoCropTEditChange(TObject *Sender)
{
    if (!S4_IsActive)
        return;

    if (!VideoCropTEdit->Text.IsEmpty())
    {
        try
        {
            VideoCropTEdit->Text.ToInt();
        }
        catch (...)
        {
            return;
        }

        CropHeight += CropTop;
        CropTop     = VideoCropTUpDown->Position;
        CropHeight -= CropTop;

        VideoCropTUpDown->Max = TitleInfo[VTS].Video.Height - VideoCropBUpDown->Position - FSK_MIN_VIDEO_RES;
        VideoCropBUpDown->Max = TitleInfo[VTS].Video.Height - VideoCropTUpDown->Position - FSK_MIN_VIDEO_RES;
        VideoCropTUpDown->Refresh();
        VideoCropBUpDown->Refresh();
        S4_UpdateCrop();
    }
}

void __fastcall TFSKNewForm::VideoCropBEditChange(TObject *Sender)
{
    if (!S4_IsActive)
        return;

    if (!VideoCropBEdit->Text.IsEmpty())
    {
        try
        {
            VideoCropBEdit->Text.ToInt();
        }
        catch (...)
        {
            return;
        }

        CropHeight = TitleInfo[VTS].Video.Height - CropTop - VideoCropBUpDown->Position;

        VideoCropTUpDown->Max = TitleInfo[VTS].Video.Height - VideoCropBUpDown->Position - FSK_MIN_VIDEO_RES;
        VideoCropBUpDown->Max = TitleInfo[VTS].Video.Height - VideoCropTUpDown->Position - FSK_MIN_VIDEO_RES;
        VideoCropTUpDown->Refresh();
        VideoCropBUpDown->Refresh();
        S4_UpdateCrop();
    }
}

void __fastcall TFSKNewForm::VideoCropLEditChange(TObject *Sender)
{
    if (!S4_IsActive)
        return;

    if (!VideoCropLEdit->Text.IsEmpty())
    {
        try
        {
            VideoCropLEdit->Text.ToInt();
        }
        catch (...)
        {
            return;
        }

        CropWidth += CropLeft;
        CropLeft   = VideoCropLUpDown->Position;
        CropWidth -= CropLeft;

        VideoCropLUpDown->Max = TitleInfo[VTS].Video.Width - VideoCropRUpDown->Position - FSK_MIN_VIDEO_RES;
        VideoCropRUpDown->Max = TitleInfo[VTS].Video.Width - VideoCropLUpDown->Position - FSK_MIN_VIDEO_RES;
        VideoCropLUpDown->Refresh();
        VideoCropRUpDown->Refresh();
        S4_UpdateCrop();
    }
}


void __fastcall TFSKNewForm::VideoCropREditChange(TObject *Sender)
{
    if (!S4_IsActive)
        return;

    if (!VideoCropREdit->Text.IsEmpty())
    {
        try
        {
            VideoCropREdit->Text.ToInt();
        }
        catch (...)
        {
            return;
        }

        CropWidth = TitleInfo[VTS].Video.Width - CropLeft - VideoCropRUpDown->Position;

        VideoCropLUpDown->Max = TitleInfo[VTS].Video.Width - VideoCropRUpDown->Position - FSK_MIN_VIDEO_RES;
        VideoCropRUpDown->Max = TitleInfo[VTS].Video.Width - VideoCropLUpDown->Position - FSK_MIN_VIDEO_RES;
        VideoCropLUpDown->Refresh();
        VideoCropRUpDown->Refresh();
        S4_UpdateCrop();
    }
}

void __fastcall TFSKNewForm::VideoCropTEditExit(TObject *Sender)
{
    if (VideoCropTEdit->Text.IsEmpty())
    {
        Application->MessageBoxA("Please enter an integer.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
        VideoCropTEdit->SetFocus();
        return;
    }
    else
    {
        try
        {
            VideoCropTEdit->Text.ToInt();
        }
        catch (...)
        {
            Application->MessageBoxA("Please enter a valid integer.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
            VideoCropTEdit->SetFocus();
            return;
        }

        VideoCropTEdit->Text = (VideoCropTUpDown->Position / 2) * 2;
    }
}

void __fastcall TFSKNewForm::VideoCropBEditExit(TObject *Sender)
{
    if (VideoCropBEdit->Text.IsEmpty())
    {
        Application->MessageBoxA("Please enter an integer.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
        VideoCropBEdit->SetFocus();
        return;
    }
    else
    {
        try
        {
            VideoCropBEdit->Text.ToInt();
        }
        catch (...)
        {
            Application->MessageBoxA("Please enter a valid integer.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
            VideoCropBEdit->SetFocus();
            return;
        }

        VideoCropBEdit->Text = (VideoCropBUpDown->Position / 2) * 2;
    }
}

void __fastcall TFSKNewForm::VideoCropLEditExit(TObject *Sender)
{
    if (VideoCropLEdit->Text.IsEmpty())
    {
        Application->MessageBoxA("Please enter an integer.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
        VideoCropLEdit->SetFocus();
        return;
    }
    else
    {
        try
        {
            VideoCropLEdit->Text.ToInt();
        }
        catch (...)
        {
            Application->MessageBoxA("Please enter a valid integer.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
            VideoCropLEdit->SetFocus();
            return;
        }

        VideoCropLEdit->Text = (VideoCropLUpDown->Position / 2) * 2;
    }
}

void __fastcall TFSKNewForm::VideoCropREditExit(TObject *Sender)
{
    if (VideoCropREdit->Text.IsEmpty())
    {
        Application->MessageBoxA("Please enter an integer.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
        VideoCropREdit->SetFocus();
        return;
    }
    else
    {
        try
        {
            VideoCropREdit->Text.ToInt();
        }
        catch (...)
        {
            Application->MessageBoxA("Please enter a valid integer.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
            VideoCropREdit->SetFocus();
            return;
        }

        VideoCropREdit->Text = (VideoCropRUpDown->Position / 2) * 2;
    }
}

void __fastcall TFSKNewForm::VideoCropAdjButtonClick(TObject *Sender)
{
    FSKNewCropForm = new TFSKNewCropForm(FSKMenuForm);

    if (!FSKNewCropForm->Init())
    {
        // Error
        Application->MessageBoxA("Unable to view selected title!",
                                 FSK_PRODUCT_NAME, MB_ICONERROR | MB_OK);
    }
    else
        FSKNewCropForm->Show();
}

void __fastcall TFSKNewForm::VideoTruCheckClick(TObject *Sender)
{
    AnsiString Text;

    if (!S4_IsActive)
        return;

    if (VideoTruCheck->Checked)
        // Change framerate to 24(Film)
        Framerate = 24;
    else
        // Reset framerate
        Framerate = OriFramerate;

    // Update controls
    Text.printf("%.2f fps", float(int(Framerate * 100)) / 100);
    VideoInfoList->Items->Item[3]->SubItems->Strings[0] = Text;
    S4_UpdateDuration();
    S4_UpdateBitrate();
}

void __fastcall TFSKNewForm::VideoFiltersButtonClick(TObject *Sender)
{
    FSKNewFiltersForm = new TFSKNewFiltersForm(FSKMenuForm);
    FSKNewFiltersForm->Show();
}

void __fastcall TFSKNewForm::VideoCodecButtonClick(TObject *Sender)
{
    FSKNewCodecForm = new TFSKNewCodecForm(FSKMenuForm);
    FSKNewCodecForm->Show();
}

/*---------------------------------------------------------------------------*/

void __fastcall TFSKNewForm::Audio1TrackComboChange(TObject *Sender)
{
    if (!S4_IsActive)
        return;

    // Update bitrate
    S4_UpdateBitrate(false, true);
}

void __fastcall TFSKNewForm::Audio1EncModeRadioClick(TObject *Sender)
{
    if (!S4_IsActive)
        return;

    // Update bitrate
    S4_UpdateBitrate(false, true);
}

void __fastcall TFSKNewForm::Audio1BitrateRadioClick(TObject *Sender)
{
    if (!S4_IsActive)
        return;

    // Enable/disable bitrate slider
    Audio1BitrateTrack->Enabled = !Audio1BitrateAutoRadio->Checked && Audio1BitrateTrack->Max;

    // Update bitrate (if required)
    if (Audio1BitrateAutoRadio->Checked)
        S4_UpdateBitrate();
}

void __fastcall TFSKNewForm::Audio1BitrateTrackChange(TObject *Sender)
{
    if (!S4_IsActive)
        return;

    // Update bitrate
    S4_UpdateBitrate(false, false);
}

/*---------------------------------------------------------------------------*/

void __fastcall TFSKNewForm::SubCheckClick(TObject *Sender)
{
    if (!S4_IsActive)
        return;

    if (SubCheck->Checked)
    {
        SubLabel1->Enabled    = true;
        SubLangCombo->Enabled = true;

        if (VideoCropAutoCheck->Checked)
        {
            S4_IsActive = false;
            VideoCropNoCheck->Checked = true;
            S4_IsActive = true;
            VideoCropNoCheckClick(Sender);
        }

        VideoCropAutoCheck->Enabled = false;
    }
    else
    {
        SubLabel1->Enabled    = false;
        SubLangCombo->Enabled = false;

        VideoCropAutoCheck->Enabled = TitleInfo[VTS].PGC[PGC].Video.Crop > 0;
    }
}

/*---------------------------------------------------------------------------*/

void __fastcall TFSKNewForm::InfoEditKeyPress(TObject *Sender, char &Key)
{
    // Ignore invalid characters
    if (!isascii(Key) || Key == '&' || Key == '<' || Key == '>' || Key == '"' || Key == '\'')
    {
        Key = 0; // No key pressed
        MessageBeep(MB_OK);
    }
}

void __fastcall TFSKNewForm::InfoTitleEditExit(TObject *Sender)
{
    for (int i = 1; i <= InfoTitleEdit->Text.Length(); i++)
        if (!isascii(InfoTitleEdit->Text[i]) ||
            InfoTitleEdit->Text[i] == '&' || InfoTitleEdit->Text[i] == '<' ||
            InfoTitleEdit->Text[i] == '>' || InfoTitleEdit->Text[i] == '"' ||
            InfoTitleEdit->Text[i] == '\'')
        {
            Application->MessageBoxA("This text contains invalid characters. Please enter a valid text!", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
            InfoTitleEdit->SetFocus();
            return;
        }
}

void __fastcall TFSKNewForm::InfoAuthorEditExit(TObject *Sender)
{
    for (int i = 1; i <= InfoAuthorEdit->Text.Length(); i++)
        if (!isascii(InfoAuthorEdit->Text[i]) ||
            InfoAuthorEdit->Text[i] == '&' || InfoAuthorEdit->Text[i] == '<' ||
            InfoAuthorEdit->Text[i] == '>' || InfoAuthorEdit->Text[i] == '"' ||
            InfoAuthorEdit->Text[i] == '\'')
        {
            Application->MessageBoxA("This text contains invalid characters. Please enter a valid text!", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
            InfoAuthorEdit->SetFocus();
            return;
        }
}

void __fastcall TFSKNewForm::InfoCopyEditExit(TObject *Sender)
{
    for (int i = 1; i <= InfoCopyEdit->Text.Length(); i++)
        if (!isascii(InfoCopyEdit->Text[i]) ||
            InfoCopyEdit->Text[i] == '&' || InfoCopyEdit->Text[i] == '<' ||
            InfoCopyEdit->Text[i] == '>' || InfoCopyEdit->Text[i] == '"' ||
            InfoCopyEdit->Text[i] == '\'')
        {
            Application->MessageBoxA("This text contains invalid characters. Please enter a valid text!", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
            InfoCopyEdit->SetFocus();
            return;
        }
}

void __fastcall TFSKNewForm::InfoWordsEditExit(TObject *Sender)
{
    for (int i = 1; i <= InfoWordsEdit->Text.Length(); i++)
        if (!isascii(InfoWordsEdit->Text[i]) ||
            InfoWordsEdit->Text[i] == '&' || InfoWordsEdit->Text[i] == '<' ||
            InfoWordsEdit->Text[i] == '>' || InfoWordsEdit->Text[i] == '"' ||
            InfoWordsEdit->Text[i] == '\'')
        {
            Application->MessageBoxA("This text contains invalid characters. Please enter a valid text!", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
            InfoWordsEdit->SetFocus();
            return;
        }
}

void __fastcall TFSKNewForm::InfoDescMemoExit(TObject *Sender)
{
    for (int i = 1; i <= InfoDescMemo->Text.Length(); i++)
        if (!isascii(InfoDescMemo->Text[i]) ||
            InfoDescMemo->Text[i] == '&' || InfoDescMemo->Text[i] == '<' ||
            InfoDescMemo->Text[i] == '>' || InfoDescMemo->Text[i] == '"' ||
            InfoDescMemo->Text[i] == '\'')
        {
            Application->MessageBoxA("This text contains invalid characters. Please enter a valid text!", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
            InfoDescMemo->SetFocus();
            return;
        }
}

/*---------------------------------------------------------------------------*/

void __fastcall TFSKNewForm::PreviewButtonClick(TObject *Sender)
{
    // Create the preview script and play it
    if (!S4_MakeAVS(true) ||
        (int)ShellExecute(FSKMainForm->Handle, "open", (FSKAppPath + "mplayer2.exe").c_str(), "preview.avs", JobPath.c_str(), SW_SHOW) <= 32)
    {
        // Error
        Application->MessageBoxA("Unable to preview selected title!",
                                 FSK_PRODUCT_NAME, MB_ICONERROR | MB_OK);
    }
}

/*-----------------------------------------------------------------------------
    TFSKNewForm >> Step 4 >> User methods
-----------------------------------------------------------------------------*/

void TFSKNewForm::S4_Begin()
{
    int         i;       // counter
    int         x;       // temp integer
    AnsiString  Text;    // temp string
    char       *Lang;    // language string

    // Set header icon
    HeaderIconList->GetIcon(3, HeaderIcon->Picture->Icon);

    // Set header text
    HeaderTitle->Caption   = "Configuration";
    HeaderComment->Caption = "Set transcoding properties.";

    // Audio gain ticks
    for (i = -12; i <= 12; i++)
        Audio1GainTrack->SetTick(i * 5);

    // Reset stuff if required
    if (S4_Reset)
    {
        S4_IsActive = false;

        // Store current selection
        S3_PrevVTS   = VTS;
        S3_PrevPGC   = PGC;
        S3_PrevAngle = Angle;

        AudioPresets    = 0;
        VideoComplexity = 0;

        // Get framerate
        Framerate = OriFramerate = TitleInfo[VTS].Video.Framerate;

        // Get duration
        OriNumFrames = NumFrames = TitleInfo[VTS].PGC[PGC].Video.NumFrames;
        S4_UpdateDuration();
        OriNumSeconds = S4_FramesToSeconds(NumFrames, OriFramerate);

        // Set video page as active
        StreamPageControl->ActivePageIndex = 0;

        // Video info: System
        if (TitleInfo[VTS].Video.IsPAL)
            Text = "PAL";
        else
            Text = "NTSC";

        VideoInfoList->Items->Item[0]->SubItems->Strings[0] = Text;

        // Video info: Resolution
        Text.printf("%dx%d", TitleInfo[VTS].Video.Width, TitleInfo[VTS].Video.Height);
        VideoInfoList->Items->Item[1]->SubItems->Strings[0] = Text;

        // Video info: Display
        if (TitleInfo[VTS].Video.IsWide)
            VideoInfoList->Items->Item[2]->SubItems->Strings[0] = "16:9 Wide";
        else
            VideoInfoList->Items->Item[2]->SubItems->Strings[0] = "4:3";

        // Video info: Frame rate
        Text.printf("%.2f fps", float(int(Framerate * 100)) / 100);
        VideoInfoList->Items->Item[3]->SubItems->Strings[0] = Text;

        // PAL TruSpeed
        VideoTruCheck->Enabled = TitleInfo[VTS].Video.IsPAL;

        // Video info: Frame type
        if (TitleInfo[VTS].Video.IsProgressive)
            VideoInfoList->Items->Item[4]->SubItems->Strings[0] = "Progressive";
        else
            VideoInfoList->Items->Item[4]->SubItems->Strings[0] = "Interlaced";

        // Audio info
        Audio1TrackCombo->Clear();

        if (TitleInfo[VTS].PGC[PGC].AudioNum)
        {
            for (i = 0; i < TitleInfo[VTS].PGC[PGC].AudioNum; i++)
            {
                Text.printf("%d. %s ", i + 1, FSKAudioCodecs[TitleInfo[VTS].PGC[PGC].Audio[i].Format].Name);
                if (TitleInfo[VTS].PGC[PGC].Audio[i].Channels == 6 || TitleInfo[VTS].PGC[PGC].Audio[i].Channels == 8)
                    Text.cat_printf("%d.1-ch ", TitleInfo[VTS].PGC[PGC].Audio[i].Channels - 1);
                else
                    Text.cat_printf("%d-ch ", TitleInfo[VTS].PGC[PGC].Audio[i].Channels);

                Lang = DecodeLanguage(TitleInfo[VTS].PGC[PGC].Audio[i].Language);
                if (strcmp(Lang, "Unknown"))
                    Text.cat_printf("%s ", DecodeLanguage(TitleInfo[VTS].PGC[PGC].Audio[i].Language));

                if (TitleInfo[VTS].PGC[PGC].Audio[i].Caption == 3)
                    Text.cat_printf("(Commentary)");

                x = FSKAudioCodecs[TitleInfo[VTS].PGC[PGC].Audio[i].Format].IconIndex;
                Audio1TrackCombo->ItemsEx->AddItem(Text, x, x, x, 0, NULL);
            }

            AudioSheet->TabVisible = true;
            IsAudio = true;
        }
        else
        {
            AudioSheet->TabVisible = false;
            IsAudio = false;
        }

        // Subpicture info
        SubLangCombo->Clear();

        if (TitleInfo[VTS].PGC[PGC].SubpicNum)
        {
            for (i = 0; i < TitleInfo[VTS].PGC[PGC].SubpicNum; i++)
            {
                Text.printf("%d. %s", i + 1, DecodeLanguage(TitleInfo[VTS].PGC[PGC].Subpic[i].Language));
                SubLangCombo->Items->Add(Text);
            }

            SubSheet->TabVisible = true;
            IsSubpic = true;
        }
        else
        {
            SubSheet->TabVisible = false;
            IsSubpic = false;
        }

        // Default configuration
        S4_SetDefault();
    }
    else
    {
        // Restore the "Next" button previous state
        NextButton->Enabled = S4_DoneButtonEnabled;
    }

    // Switch to the page in the notebook
    Notebook->PageIndex = 3;

    // Change the "Next" button's caption
    NextButton->Caption     = "Finish";
    NextButton->Font->Style = TFontStyles() << fsBold;

    // Show preview button if required
    PreviewButton->Visible = NextButton->Enabled;
}

bool TFSKNewForm::S4_End(bool Forward)
{
    FILE       *File;           // file handle
    AnsiString  FilePath;       // file path
    int         VolumeCount;    // number of volumes
    bool        Exists;

    if (Forward)
    {
        Screen->Cursor = crHourGlass;

        // Make job files
        if (!S4_MakeAVS(false) || !S4_MakeRPJF())
        {
            RemoveDirTree(JobRoot);
            Application->MessageBoxA("Unable to save job!", FSK_PRODUCT_NAME, MB_ICONERROR | MB_OK);
            goto End;
        }

        // Get volume count
        if (EncModeFixedRadio->Checked && VolCheck->Checked)
            VolumeCount = VolNumUpDown->Position;
        else
            VolumeCount = 1;

        // Write job info file
        FilePath = JobRoot + FSKJ_INFO_FILENAME;
        if ((File = fopen(FilePath.c_str(), "wt")) == NULL)
        {
            RemoveDirTree(JobRoot);
            Application->MessageBoxA("Unable to save job!", FSK_PRODUCT_NAME, MB_ICONERROR | MB_OK);
            goto End;
        }

        fprintf(File, FSKJ_ID);
        fprintf(File, "\n\n%s\n%s\n0 / %d\n--\n--",
                JobName,
                Now().DateTimeString().c_str(),
                VolumeCount);

        fclose(File);

        // Add job to the list
        if (FSKMenuForm->AddJobA(JobRoot, Exists))
        {
            if (Exists)
                Application->MessageBoxA("Unable to add job to the list!",
                                         FSK_PRODUCT_NAME, MB_ICONEXCLAMATION | MB_OK);
        }
        else
            Application->MessageBoxA("Job is corrupted!",
                                     FSK_PRODUCT_NAME, MB_ICONERROR | MB_OK);

        // End
End:
        Screen->Cursor = crDefault;
        FSKMenuForm->Enabled = true;
        Release();
    }
    else
    {
        // We don't have to reset next time
        S4_Reset = false;

        // Store "Next" button state
        S4_DoneButtonEnabled = NextButton->Enabled;

        // Reset the "Next" button's caption
        NextButton->Font->Style = TFontStyles();
        NextButton->Caption     = "Next >";

        // Hide preview button
        PreviewButton->Visible = false;
    }

    return true;
}

/*---------------------------------------------------------------------------*/

// Set default settings
void TFSKNewForm::S4_SetDefault()
{
    S4_IsActive = false;

    // Initialization
    Bitrate = 0;

    // Output size
    EncModeFixedRadio->Checked = true;

    // Video
    if (TitleInfo[VTS].PGC[PGC].Video.Crop > 0)
    {
        VideoCropAutoCheck->Checked = true;
        VideoCropNoCheck->Enabled = true;
        VideoCropAutoCheck->Enabled = true;
        VideoCropManuCheck->Enabled = true;

        CropTop    = TitleInfo[VTS].PGC[PGC].Video.Top;
        CropLeft   = TitleInfo[VTS].PGC[PGC].Video.Left;
        CropWidth  = TitleInfo[VTS].PGC[PGC].Video.Width;
        CropHeight = TitleInfo[VTS].PGC[PGC].Video.Height;
    }
    else
    {
        VideoCropNoCheck->Checked = true;
        VideoCropNoCheck->Enabled = true;
        VideoCropAutoCheck->Enabled = false;
        VideoCropManuCheck->Enabled = true;

        CropTop = CropLeft = CropWidth = CropHeight = 0;
    }

    VideoResAutoCheck->Checked = true;
    VideoTruCheck->Checked     = false;

    // Calculate aspect ratio
    VideoAR = S4_CalcVideoAR(true, true);

    // Video filters
    if (TitleInfo[VTS].Video.IsPAL)
    {
        // PAL
        if (TitleInfo[VTS].Video.IsProgressive)
        {
            // PAL Progressive
            Deinterlace  = false;
            IVTC         = false;
        }
        else
        {
            // PAL Interlaced
            Deinterlace     = true;
            FullDeinterlace = false;
            IVTC            = false;
        }

        ComplexIVTCDecimate = false;
    }
    else
    {
        // NTSC/Film
        if (TitleInfo[VTS].Video.FilmPurity <= 5)
        {
            // Pure NTSC
            if (TitleInfo[VTS].Video.IsProgressive)
            {
                // NTSC Progressive
                Deinterlace = false;
                IVTC        = false;
            }
            else
            {
                // NTSC Interlaced
                Deinterlace     = true;
                FullDeinterlace = false;
                IVTC            = false;
            }

            ComplexIVTCDecimate = false;
        }
        else
            if (TitleInfo[VTS].Video.FilmPurity >= 95)
            {
                // Pure Film
                Deinterlace  = false;
                IVTC         = true;
                ComplexIVTC  = false;
                ComplexIVTCDecimate = true;
            }
            else
                if (TitleInfo[VTS].Video.FilmPurity < 50)
                {
                    // Hybrid (mostly NTSC)
                    Deinterlace  = false;
                    IVTC         = true;
                    ComplexIVTC  = true;
                    ComplexIVTCDecimate = false;
                }
                else
                {
                    // Hybrid (mostly Film)
                    Deinterlace  = false;
                    IVTC         = true;
                    ComplexIVTC  = true;
                    ComplexIVTCDecimate = true;
                }
    }

    IsTFF = TitleInfo[VTS].Video.IsTFF;

    S4_UpdateDecimate();

    // Set recommanded output size
    double Bitrate;
    int    Size, VolSize, VolCount;

    AudioBitrate = NULL;

    VideoBitrate    = Framerate * TitleInfo[VTS].PGC[PGC].Video.Width * TitleInfo[VTS].PGC[PGC].Video.Width / S4_CalcVideoAR(true, true) * FSK_VIDEO_BPP;
    OriVideoBitrate = OriFramerate * TitleInfo[VTS].PGC[PGC].Video.Width * TitleInfo[VTS].PGC[PGC].Video.Width / S4_CalcVideoAR(true, true) * FSK_VIDEO_BPP;
    if (IsAudio)
        Bitrate  = VideoBitrate / (1 - FSK_AUDIO_BITRATE_RATIO) + (32 * 1024);   // just to be sure :)
    else
        Bitrate  = VideoBitrate;

    EncModeBitrateEdit->Text = IntToStr(int(Bitrate / 1024));
    EncModeQualityEdit->Text = IntToStr(85);   // 85 %

    Size    = int(ceil((Bitrate * NumSeconds) / (1024 * 1024 * 8)));
    VolSize = FSKVolumeSize[FSK_DEFAULT_VOLUME_IDX - 1].Size;

    if (Size < (VolSize / 2))
    {
        // Resulted size is significantly smaller than volume size
        VolCombo->ItemIndex = 0;
        VolEdit->Text = IntToStr(Size);
        VolNumEdit->Text = IntToStr(1);
    }
    else
    {
        // Calculate number of volumes
        VolCombo->ItemIndex = FSK_DEFAULT_VOLUME_IDX;
        VolEdit->Text = IntToStr(FSKVolumeSize[VolCombo->ItemIndex - 1].Size);

        if ((Size % VolSize) < (VolSize / 2))
            VolCount = Size / VolSize;
        else
            VolCount = (Size / VolSize) + 1;

        VolNumEdit->Text = IntToStr(VolCount);
    }

    // Audio
    PrevAudioPresets = AudioPresets = NULL;

    if (IsAudio)
    {
        // Select first track
        Audio1TrackCombo->ItemIndex = 0;
        Audio1BitrateAutoRadio->Checked = true;
    }

    // Disable gain
    Audio1GainCheck->Checked  = false;
    Audio1GainTrack->Position = 0;

    // Subpicture
    if (IsSubpic)
    {
        // Select first track
        SubLangCombo->ItemIndex = 0;

        // Disable subtitles
        SubCheck->Checked = false;
    }

    // Finalization
    S4_IsActive = true;

    if (IsAudio)
        Audio1TrackComboChange(NULL);

    VideoResAutoCheckClick(NULL);

    if (TitleInfo[VTS].PGC[PGC].Video.Crop)
        VideoCropAutoCheckClick(NULL);
    else
        VideoCropNoCheckClick(NULL);

    if (IsAudio)
        Audio1BitrateRadioClick(NULL);

    Audio1GainCheckClick(NULL);

    EncModeFixedRadioClick(NULL);
    VolComboChange(NULL);
    VolNumEditChange(NULL);

    if (IsSubpic)
        SubCheckClick(NULL);

    S4_IsActive = false;
    EHQCustomCheck->Checked = false;
    EHQTrack->Position = EHQTrack->Max - 1;
    S4_IsActive = true;
    EHQCustomCheckClick(NULL);
    S4_UpdateVideoCodec();

    NextButton->Enabled    = true;
    PreviewButton->Visible = true;

    Refresh();
}

// Calculate video aspect ratio
void TFSKNewForm::S4_CalcVideoAR()
{
    VideoAR = S4_CalcVideoAR(!VideoCropNoCheck->Checked, false);
}

double TFSKNewForm::S4_CalcVideoAR(bool Crop, bool Auto)
{
    if (TitleInfo[VTS].Video.IsWide)
    {
        if (Crop && TitleInfo[VTS].PGC[PGC].Video.Crop)
        {
            if (Auto)
                return double(TitleInfo[VTS].PGC[PGC].Video.Width * TitleInfo[VTS].Video.Height * 16) / (TitleInfo[VTS].Video.Width * TitleInfo[VTS].PGC[PGC].Video.Height * 9);
            else
                return double(CropWidth * TitleInfo[VTS].Video.Height * 16) / (TitleInfo[VTS].Video.Width * CropHeight * 9);
        }
        else
            return 16.0 / 9;
    }
    else
    {
        if (Crop && TitleInfo[VTS].PGC[PGC].Video.Crop)
        {
            if (Auto)
                return double(TitleInfo[VTS].PGC[PGC].Video.Width * TitleInfo[VTS].Video.Height * 4) / (TitleInfo[VTS].Video.Width * TitleInfo[VTS].PGC[PGC].Video.Height * 3);
            else
                return double(CropWidth * TitleInfo[VTS].Video.Height * 4) / (TitleInfo[VTS].Video.Width * CropHeight * 3);
        }
        else
            return 4.0 / 3;
    }
}

// Adjust the resolution (make the width and height multiples of 4)
void TFSKNewForm::S4_AdjustResolution(int& Width, int& Height, double AR, bool WRef)
{
    // Calculate height
    if (WRef)
        Height = int(ceil(double(Width) / AR));
    else
        Width = int(floor((double)Height * AR));

    // Check resolution
    if (Width < FSK_MIN_VIDEO_RES || Height < FSK_MIN_VIDEO_RES)
    {
        if (AR >= 1)
        {
            Height = FSK_MIN_VIDEO_RES;
            Width  = int(floor(double(Height) * AR));
        }
        else
        {
            Width  = FSK_MIN_VIDEO_RES;
            Height = int(ceil(double(Width) / AR));
        }
    }
    else
        if (Width > FSK_MAX_VIDEO_RES || Height > FSK_MAX_VIDEO_RES)
        {
            if (AR >= 1)
            {
                Width  = FSK_MAX_VIDEO_RES;
                Height = int(ceil(double(Width) / AR));
            }
            else
            {
                Height = FSK_MAX_VIDEO_RES;
                Width  = int(floor(double(Height) * AR));
            }
        }

    if (WRef)
    {
        // Adjust width
        if ((Width % 4) < 2)
            Width = (Width / 4) * 4;
        else
            Width = (Width / 4 + 1) * 4;

        // Recalculate height
        Height = int(ceil(double(Width) / AR));

        // Adjust height
        if ((Height % 4) < 2)
            Height = (Height / 4) * 4;
        else
            Height = (Height / 4 + 1) * 4;
    }
    else
    {
        // Adjust height
        if ((Height % 4) < 2)
            Height = (Height / 4) * 4;
        else
            Height = (Height / 4 + 1) * 4;

        // Recalculate width
        Width = int(floor((double)Height * AR));

        // Adjust width
        if ((Width % 4) < 2)
            Width = (Width / 4) * 4;
        else
            Width = (Width / 4 + 1) * 4;
    }
}

// Convert frame count/framerate to seconds
double TFSKNewForm::S4_FramesToSeconds(int Frames, float Framerate)
{
    return double(Frames) / Framerate;
}

// Update title duration
void TFSKNewForm::S4_UpdateDuration()
{
    AnsiString Text;                        // text buffer
    double     _NumSeconds;                 // temporary duration is seconds
    int        Hours, Minutes, Seconds;     // timecode

    // Recalculate number of seconds
    NumSeconds = S4_FramesToSeconds(NumFrames, Framerate);

    // Calculate timecode
    _NumSeconds = NumSeconds;

    Hours   = int(_NumSeconds / 3600); _NumSeconds -= Hours * 3600;
    Minutes = int(_NumSeconds / 60);   _NumSeconds -= Minutes * 60;
    Seconds = int(_NumSeconds);

    // Update duration indicator (GUI)
    Text.printf("%.2d:%.2d:%.2d", Hours, Minutes, Seconds);
    VideoInfoList->Items->Item[5]->SubItems->Strings[0] = Text;
}

// Update framerate (IVTC decimation)
void TFSKNewForm::S4_UpdateDecimate()
{
    AnsiString Text;

    // Update framerate and framecount
    if (IVTC && (!ComplexIVTC || (ComplexIVTC && ComplexIVTCDecimate)))
    {
        Framerate = (OriFramerate * 4) / 5;
        NumFrames = (OriNumFrames * 4) / 5;
    }
    else
    {
        Framerate = OriFramerate;
        NumFrames = OriNumFrames;
    }

    // Update GUI
    Text.printf("%.2f fps", float(int(Framerate * 100)) / 100);
    VideoInfoList->Items->Item[3]->SubItems->Strings[0] = Text;

    S4_UpdateDuration();    // just to be sure because of rounding errors
}

// Recalculate bitrate and auto stuff
void TFSKNewForm::S4_UpdateBitrate(bool Full, bool UpdateAudioMode)
{
    // Override Full flag if necessary
    if (!Bitrate)
        Full = true;

    S4_IsActive = false;

    if (Full)
    {
        if (EncModeFixedRadio->Checked)
        {
            Bitrate    = ((double(VolUpDown->Position) * VolNumUpDown->Position * 1024 * 1024 * 8) / NumSeconds) * FSK_OVERHEAD;
            OriBitrate = ((double(VolUpDown->Position) * VolNumUpDown->Position * 1024 * 1024 * 8) / OriNumSeconds) * FSK_OVERHEAD;

            if (Bitrate > EncModeBitrateUpDown->Max * 1024)
                Bitrate = double(EncModeBitrateUpDown->Max * 1024);

            if (OriBitrate > EncModeBitrateUpDown->Max * 1024)
                OriBitrate = double(EncModeBitrateUpDown->Max * 1024);

            EncModeBitrateEdit->Text = IntToStr(int(Bitrate / 1024));
        }
        else
        {
            if (EncModeBitrateRadio->Checked)
            {
                Bitrate    = double(EncModeBitrateUpDown->Position) * 1024;
                OriBitrate = Bitrate;

                // Update estimated number of volumes if "Average bitrate" is selected
                if (VolCheck->Checked)
                    VolNumEdit->Text = IntToStr(int(ceil((OriBitrate * OriNumSeconds) / (double(VolUpDown->Position) * 1024 * 1024 * 8))));
            }
            else
            {
                Bitrate = OriBitrate = -1;
            }
        }
    }

    // Update audio bitrate
    if (IsAudio)
    {
        S4_IsActive = true;

        if (Full || UpdateAudioMode)
            S4_UpdateAudioMode();
        else
            S4_UpdateAudioBitrate(false);

        S4_IsActive = false;
    }
    else
    {
        AudioBitrate = NULL;

        // Update minimum bitrate and output size
        if (EncModeFixedRadio->Checked || EncModeBitrateRadio->Checked)
            EncModeBitrateUpDown->Min = FSK_MIN_VIDEO_BITRATE;

        if (EncModeFixedRadio->Checked)
        {
            S4_MinOutputSize = int(ceil(double(FSK_MIN_VIDEO_BITRATE) * OriNumSeconds / (1024 * 8)));
            if (!S4_MinOutputSize)
                S4_MinOutputSize = 1;
            VolUpDown->Min = int(ceil(double(S4_MinOutputSize) / VolNumUpDown->Position));
        }
        else
        {
            S4_MinOutputSize = 1;
            VolUpDown->Min = 1;
        }
    }

    // Return if in quality mode
    if (EncModeQualityRadio->Checked)
    {
        S4_IsActive = true;
        return;
    }

    // Video bitrate and resolution
    VideoBitrate    = Bitrate - AudioBitrate;
    OriVideoBitrate = OriBitrate - AudioBitrate;

    // Update automatic frame size
    S4_IsActive = true;

    S4_UpdateAutoResize();
}

// Update automatic resized resolution
void TFSKNewForm::S4_UpdateAutoResize()
{
    double       AR;                    // bitrate and aspect ratio
    double       _Width, _Height;       // resolution (floating point)
    int          Width, Height;         // resolution
    int          OriWidth, OriHeight;   // original resolution

    if (!VideoResAutoCheck->Checked || !VideoAR || !Framerate || (NumSeconds == 0.0) || (VideoBitrate <= 0))
        return;

    S4_IsActive = false;

    // Get original resolution
    if (!VideoCropNoCheck->Checked)
    {
        OriWidth  = CropWidth;
        OriHeight = CropHeight;
    }
    else
    {
        OriWidth  = TitleInfo[VTS].Video.Width;
        OriHeight = TitleInfo[VTS].Video.Height;
    }

    // Get aspect ratio
    AR = S4_CalcVideoAR(true, true);

    // Calculate resolution
    _Height = sqrt(OriVideoBitrate / (FSK_VIDEO_BPP * OriFramerate * AR));
    _Width  = AR * _Height;

    if (TitleInfo[VTS].PGC[PGC].Video.Crop && VideoCropNoCheck->Checked)
    {
        _Width = (_Width * OriWidth) / TitleInfo[VTS].PGC[PGC].Video.Width;
        _Height = _Width / VideoAR;
    }

    Width  = int(_Width);
    Height = int(_Height);

    // Check if the resulted size exceeds the original size
    if (Width > OriWidth || Height > OriHeight)
    {
        if (VideoAR >= 1)
        {
            Width  = OriWidth;
            Height = int(ceil((double)Width / VideoAR));
        }
        else
        {
            Height = OriHeight;
            Width  = int(floor(double(Height) * VideoAR));
        }
    }

    S4_AdjustResolution(Width, Height, VideoAR, true);

    // Update indicator (GUI)
    VideoResWEdit->Text = IntToStr(Width);
    VideoResHEdit->Text = IntToStr(Height);

    S4_IsActive = true;
}

void TFSKNewForm::S4_UpdateCrop()
{
    // Update resolution
    S4_CalcVideoAR();

    if (VideoResAutoCheck->Checked)
        S4_UpdateAutoResize();
    else
    {
        int Width, Height;

        S4_IsActive = false;

        // Calculate height
        Width = VideoResWUpDown->Position;
        if (Width)
            Height = int(ceil((double)Width / VideoAR));
        else
            Height = 0;

        // Adjust resolution
        S4_AdjustResolution(Width, Height, VideoAR, true);
        VideoResWEdit->Text = IntToStr(Width);
        VideoResHEdit->Text = IntToStr(Height);

        S4_IsActive = true;
    }
}

// Update video codec options
void TFSKNewForm::S4_UpdateVideoCodec()
{
    // Disable curve compression in one pass mode
    if (!EHQTwoPassCheck->Checked)
        rcEnableCurveCompression = false;

    // Automatic adjustments
    if (!ManualVideoCodec)
    {
        if (EHQTwoPassCheck->Checked)
            rcEnableCurveCompression = true;

        rcHighBitrateReduce = 0;
        rcLowBitrateBoost   = 0;

        rcPFrameRefQuant = 6;
        rcBFrameRefQuant = 10;

        inloopCutOffQuant        = 12;
        inloopCutOffCompatible   = false;
        inloopCutOffBUseRefQuant = false;

        AdaptiveB             = true;
        maxConsecutiveBFrames = 3;
        patternAdaptivity     = 2;

        encodeAllFrames    = true;
        chromaModeDecision = true;
        enableRPR          = true;
    }
}

// Update audio mode
void TFSKNewForm::S4_UpdateAudioMode()
{
    int   Count = 0;                          // number of presets in the array
    bool  Reset;

    S4_IsActive = false;

    // Store current presets
    PrevAudioPresets = AudioPresets;

    // Enable available modes
    Audio1MonoRadio->Enabled     = EncModeQualityRadio->Checked || (int(ceil(Bitrate / 1024)) >= (FSKAudioMonoPresets[0].Bitrate + FSK_MIN_VIDEO_BITRATE));
    Audio1StereoRadio->Enabled   = EncModeQualityRadio->Checked || (int(ceil(Bitrate / 1024)) >= (FSKAudioStereoPresets[0].Bitrate + FSK_MIN_VIDEO_BITRATE));
    Audio1SurroundRadio->Enabled = (TitleInfo[VTS].PGC[PGC].Audio[Audio1TrackCombo->ItemIndex].Channels > 1) && (EncModeQualityRadio->Checked || (int(ceil(Bitrate / 1024)) >= (FSKAudioSurroundPresets[0].Bitrate + FSK_MIN_VIDEO_BITRATE)));
    Audio1MultiRadio->Enabled    = (TitleInfo[VTS].PGC[PGC].Audio[Audio1TrackCombo->ItemIndex].Channels > 3) && (EncModeQualityRadio->Checked || (int(ceil(Bitrate / 1024)) >= (FSKAudioMultiPresets[0].Bitrate + FSK_MIN_VIDEO_BITRATE)));

    // Adjust mode if required
    if (Audio1SurroundRadio->Checked && !Audio1SurroundRadio->Enabled)
        Audio1StereoRadio->Checked = true;
    else
        if (Audio1MultiRadio->Checked && !Audio1MultiRadio->Enabled)
        {
            if (Audio1SurroundRadio->Enabled)
                Audio1SurroundRadio->Checked = true;
            else
                Audio1StereoRadio->Checked = true;
        }


    // Select mode
    Reset = false;

    if (AudioPresets)
    {
        if (Audio1MonoRadio->Checked)
            AudioPresets = FSKAudioMonoPresets;
        else
            if (Audio1StereoRadio->Checked)
                AudioPresets = FSKAudioStereoPresets;
            else
                if (Audio1SurroundRadio->Checked)
                    AudioPresets = FSKAudioSurroundPresets;
                else
                    if (Audio1MultiRadio->Checked)
                        AudioPresets = FSKAudioMultiPresets;
                    else
                        Reset = true;
    }
    else
        Reset = true;

    if (Reset)
    {
        // Autoselect mode
        if (Audio1MultiRadio->Enabled)
        {
            Audio1MultiRadio->Checked = true;
            AudioPresets = FSKAudioMultiPresets;
        }
        else
            if (Audio1SurroundRadio->Enabled)
            {
                Audio1SurroundRadio->Checked = true;
                AudioPresets = FSKAudioSurroundPresets;
            }
            else
            {
                Audio1StereoRadio->Checked = true;
                AudioPresets = FSKAudioStereoPresets;
            }
    }

    // Count number of subpresets
    while (AudioPresets[Count].Bitrate)
        Count++;

    AudioPresetCount = Count;

    // Update audio bitrate
    S4_IsActive = true;
    S4_UpdateAudioBitrate(true);
}

// Update automatic audio bitrate
void TFSKNewForm::S4_UpdateAudioBitrate(bool Full)
{
    int    i;                  // counters
    int    PresetID;           // audio preset index
    double Delta, MinDelta;    // audio profile bitrate delta
    int    PrevAudioBitrate;   // previous audio bitrate

    S4_IsActive = false;

    // Store previous audio bitrate
    if (PrevAudioPresets)
        PrevAudioBitrate = PrevAudioPresets[Audio1BitrateTrack->Position].Bitrate;

    // Enable unavailable audio presets
    if (Full)
    {
        if (EncModeQualityRadio->Checked)
            Audio1BitrateTrack->Max = AudioPresetCount - 1;
        else
        {
            for (i = 1; i < AudioPresetCount; i++)
                if (int(ceil(Bitrate / 1024)) < (AudioPresets[i].Bitrate + FSK_MIN_VIDEO_BITRATE))
                    break;

            Audio1BitrateTrack->Max = i - 1;
        }

        if (Audio1BitrateTrack->Max)
        {
            if (!Audio1BitrateAutoRadio->Checked)
                Audio1BitrateTrack->Enabled = true;
        }
        else
            Audio1BitrateTrack->Enabled = false;
    }

    // Get audio bitrate
    if (Audio1BitrateAutoRadio->Checked)
    {
        // Calculate recommanded audio bitrate
        AudioBitrate = Bitrate * FSK_AUDIO_BITRATE_RATIO;

        // Find the closest audio preset
        MinDelta = -1;

        for (i = 0; i <= Audio1BitrateTrack->Max; i++)
        {
            Delta = fabs(AudioBitrate - double(AudioPresets[i].Bitrate) * 1024);

            if ((Delta < MinDelta) || (MinDelta < 0))
            {
                MinDelta = Delta;
                PresetID = i;
            }
        }

        // Update trackbar
        Audio1BitrateTrack->Position = PresetID;

        // Update minimum bitrate and output size
        EncModeBitrateUpDown->Min = AudioPresets[0].Bitrate + FSK_MIN_VIDEO_BITRATE + 2;

        if (EncModeFixedRadio->Checked)
        {
            S4_MinOutputSize = int(ceil(double(AudioPresets[0].Bitrate + FSK_MIN_VIDEO_BITRATE + 2) * OriNumSeconds / (1024 * 8)));
            if (!S4_MinOutputSize)
                S4_MinOutputSize = 1;
            VolUpDown->Min = int(ceil(double(S4_MinOutputSize) / VolNumUpDown->Position));
        }
        else
        {
            S4_MinOutputSize = 1;
            VolUpDown->Min = 1;
        }
    }
    else
    {
        // Adjust trackbar position if required
        if (Full && PrevAudioPresets && (PrevAudioPresets != AudioPresets))
        {
            MinDelta = -1;

            for (i = 0; i <= Audio1BitrateTrack->Max; i++)
            {
                Delta = abs(AudioPresets[i].Bitrate - PrevAudioBitrate);

                if ((Delta < MinDelta) || (MinDelta < 0))
                {
                    MinDelta = Delta;
                    PresetID = i;
                }
            }

            // Update trackbar
            Audio1BitrateTrack->Position = PresetID;
        }
        else
            PresetID = Audio1BitrateTrack->Position;

        // Update minimum bitrate and output size
        if (EncModeFixedRadio->Checked || EncModeBitrateRadio->Checked)
            EncModeBitrateUpDown->Min = AudioPresets[PresetID].Bitrate + FSK_MIN_VIDEO_BITRATE + 2;

        if (EncModeFixedRadio->Checked)
        {
            S4_MinOutputSize = int(ceil(double(AudioPresets[PresetID].Bitrate + FSK_MIN_VIDEO_BITRATE + 2) * OriNumSeconds / (1024 * 8)));
            if (!S4_MinOutputSize)
                S4_MinOutputSize = 1;
            VolUpDown->Min = int(ceil(double(S4_MinOutputSize) / VolNumUpDown->Position));
        }
        else
        {
            S4_MinOutputSize = 1;
            VolUpDown->Min = 1;
        }
    }

    // Update audio bitrate
    AudioBitrate = double(AudioPresets[PresetID].Bitrate) * 1024;
    Audio1BitrateLabel->Caption = IntToStr(AudioPresets[PresetID].Bitrate) + " Kbps";

    S4_IsActive = true;
}

// Update video complexity trackbar if required
void TFSKNewForm::S4_UpdateVideoComplexity(const TFSKVideoComplexity *NewVideoComplexity)
{
    int i;                 // counter/number of presets
    int OldMax, PrevComplexity, CurComplexity;  // old maximum, previous/current overall complexity
    int PresetID;          // preset index
    int Delta, MinDelta;   // preset bitrate delta

    S4_IsActive = false;

    if (VideoComplexity)
    {
        PrevComplexity = VideoComplexity[EHQTrack->Position].Encoding;
        if (VideoComplexity[EHQTrack->Position].Analysis >= 0)
            PrevComplexity += VideoComplexity[EHQTrack->Position].Analysis;
    }
    else
        PrevComplexity = -1;

    VideoComplexity = NewVideoComplexity;

    // Update trackbar
    i = 0;
    while (VideoComplexity[i].Encoding >= 0)
        i++;

    OldMax = EHQTrack->Max;
    EHQTrack->Max = i - 1;

    // Adjust trackbar position
    if (PrevComplexity < 0)
        EHQTrack->Position = int(ceil((double(EHQTrack->Position) * EHQTrack->Max) / OldMax));
    else
    {
        MinDelta = -1;

        for (i = 0; i <= EHQTrack->Max; i++)
        {
            CurComplexity = VideoComplexity[i].Encoding;
            if (VideoComplexity[i].Analysis >= 0)
                CurComplexity += VideoComplexity[i].Analysis;

            Delta = abs(CurComplexity - PrevComplexity);

            if ((Delta < MinDelta) || (MinDelta < 0))
            {
                MinDelta = Delta;
                PresetID = i;
            }
        }

        EHQTrack->Position = PresetID;
    }

    S4_IsActive = true;
    EHQTrackChange(NULL);
    EHQCustomCheckClick(NULL);
}

void __fastcall TFSKNewForm::EHQCustomCheckClick(TObject *Sender)
{
    if (!S4_IsActive)
        return;

    if (EHQCustomCheck->Checked)
    {
        EHQTrack->Enabled = false;
        EHQTwoPassCheck->Enabled = !EncModeQualityRadio->Checked;

        S4_IsActive = false;
        if (EncModeQualityRadio->Checked)
            EHQTwoPassCheck->Checked = false;
        S4_IsActive = true;
        EHQTwoPassCheckClick(Sender);

        EHQ2Edit->Enabled   = true;
        EHQ2UpDown->Enabled = true;
        EHQ2Label1->Enabled = true;
        EHQ2Label2->Enabled = true;
    }
    else
    {
        EHQTrack->Enabled = true;
        EHQTwoPassCheck->Enabled = false;

        EHQ1Edit->Enabled   = false;
        EHQ1UpDown->Enabled = false;
        EHQ2Edit->Enabled   = false;
        EHQ2UpDown->Enabled = false;

        EHQTrackChange(Sender);
    }
}

void __fastcall TFSKNewForm::EHQTwoPassCheckClick(TObject *Sender)
{
    if (!S4_IsActive)
        return;

    if (EHQTwoPassCheck->Checked)
    {
        EHQ1Edit->Text = IntToStr(EHQ1UpDown->Position);

        EHQ1Edit->Enabled   = true;
        EHQ1UpDown->Enabled = true;
        EHQ1Label1->Enabled = true;
        EHQ1Label2->Enabled = true;
    }
    else
    {
        EHQ1Edit->Text = "0";

        EHQ1Edit->Enabled   = false;
        EHQ1UpDown->Enabled = false;
        EHQ1Label1->Enabled = false;
        EHQ1Label2->Enabled = false;
    }

    S4_UpdateVideoCodec();
}

void __fastcall TFSKNewForm::EHQTrackChange(TObject *Sender)
{
    if (!S4_IsActive || EHQCustomCheck->Checked)
        return;

    if (VideoComplexity[EHQTrack->Position].Analysis >= 0)
    {
        S4_IsActive = false;
        EHQTwoPassCheck->Checked = true;
        S4_IsActive = true;

        EHQ1Edit->Text = IntToStr(VideoComplexity[EHQTrack->Position].Analysis);

        EHQ1Label1->Enabled = true;
        EHQ1Label2->Enabled = true;
    }
    else
    {
        S4_IsActive = false;
        EHQTwoPassCheck->Checked = false;
        S4_IsActive = true;

        EHQ1Edit->Text = "0";

        EHQ1Label1->Enabled = false;
        EHQ1Label2->Enabled = false;
    }

    EHQ2Edit->Text = IntToStr(VideoComplexity[EHQTrack->Position].Encoding);
    S4_UpdateVideoCodec();
}

void __fastcall TFSKNewForm::EHQ1EditExit(TObject *Sender)
{
    if (!EHQ1Edit->Text.IsEmpty())
    {
        try
        {
            EHQ1Edit->Text.ToInt();
        }
        catch (...)
        {
            Application->MessageBoxA("Please enter a valid integer between 50 and 100.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
            EHQ1Edit->SetFocus();
            return;
        }

        EHQ1Edit->Text = IntToStr(EHQ1UpDown->Position);
    }
    else
    {
        Application->MessageBoxA("Please enter an integer between 50 and 100.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
        EHQ1Edit->SetFocus();
    }
}

void __fastcall TFSKNewForm::EHQ2EditExit(TObject *Sender)
{
    if (!EHQ2Edit->Text.IsEmpty())
    {
        try
        {
            EHQ2Edit->Text.ToInt();
        }
        catch (...)
        {
            Application->MessageBoxA("Please enter a valid integer between 50 and 100.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
            EHQ2Edit->SetFocus();
            return;
        }

        EHQ2Edit->Text = IntToStr(EHQ2UpDown->Position);
    }
    else
    {
        Application->MessageBoxA("Please enter an integer between 50 and 100.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
        EHQ2Edit->SetFocus();
    }
}

/*---------------------------------------------------------------------------*/

// Make the AviSynth script(s)
bool TFSKNewForm::S4_MakeAVS(bool Preview)
{
    FILE       *File = NULL;    // file handle
    AnsiString  FileName,       // filename
                NewSubFilename; // new ".sub" file's name
    int         iDCT;           // iDCT algorithm
    AnsiString  AudioDec;       // audio decoder filename
    float       AudioDelay;     // audio delay in milliseconds
    int         VolumeCount;    // number of volumes
    int         FramesPerVol;   // number of frames per volume

    // Initialization
    if (Preview)
    {
        VolumeCount = 1;
        iDCT = 6; // Skal's SSEMMX iDCT (fastest)
    }
    else
    {
        if (EncModeFixedRadio->Checked && VolCheck->Checked)
        {
            VolumeCount  = VolNumUpDown->Position;
            FramesPerVol = NumFrames / VolumeCount;
        }
        else
            VolumeCount = 1;

        iDCT = 4; // 64-bit IEEE-1180 Reference
    }

    // Volume loop
    for (int i = 1; i <= VolumeCount; i++)
    {
        // Create file
        if (Preview)
            File = fopen(PreviewPath.c_str(), "wt");
        else
        {
            FileName = JobPath;
            FileName.cat_printf(FSK_OUT_FILENAME, i);
            FileName.cat_printf(".avs");

            File = fopen(FileName.c_str(), "wt");
        }

        if (!File)
            return false;

        // Plugins
        fprintf(File, "LoadPlugin(\"%s\")\n", FSK_VIDEO_DECODER);
        if ((IVTC && ComplexIVTC) || Deinterlace)
            fprintf(File, "LoadPlugin(\"%s\")\n", FSK_VIDEO_DECOMB);

        if (IsAudio)
        {
            AudioDec = FSKAudioCodecs[TitleInfo[VTS].PGC[PGC].Audio[Audio1TrackCombo->ItemIndex].Format].DecoderName;
            fprintf(File, "LoadPlugin(\"%s\")\n", AudioDec);
        }

        if (IsSubpic && SubCheck->Checked)
            fprintf(File, "LoadPlugin(\"%s\")\n", FSK_SUBPIC_DECODER);

        fprintf(File, "\n");

        // Video
        if (IVTC && !ComplexIVTC)
            fprintf(File, "video = m2VideoSource(\"%s.m2vi\", idct=%d, force_film=true)\n", ReIndexName.c_str(), iDCT);
        else
        {
            fprintf(File, "video = m2VideoSource(\"%s.m2vi\", idct=%d, force_film=false)\n", ReIndexName.c_str(), iDCT);

            if (IVTC)
            {
                int Order;

                if (IsTFF)
                    Order = 1;
                else
                    Order = 0;

                if (TitleInfo[VTS].Video.FilmPurity >= 95)
                    // Pure Film
                    fprintf(File, "video = Telecide(video, order=%d).Decimate(cycle=5)\n", Order);
                else
                    if (TitleInfo[VTS].Video.FilmPurity < 50)
                        // Hybrid (mostly NTSC)
                        fprintf(File, "video = Telecide(video, order=%d, guide=1).Decimate(mode=1, threshold=1.0)\n", Order);
                    else
                        // Hybrid (mostly Film)
                        fprintf(File, "video = Telecide(video, order=%d, guide=1).Decimate(mode=3, threshold=1.0)\n", Order);
            }

            if (Deinterlace)
            {
                if (FullDeinterlace)
                    fprintf(File, "video = FieldDeinterlace(video, full=true)\n");
                else
                    fprintf(File, "video = FieldDeinterlace(video, full=false)\n");
            }
        }

        if (!VideoCropNoCheck->Checked &&
            (CropTop || CropLeft ||
             CropWidth != TitleInfo[VTS].Video.Width || CropHeight != TitleInfo[VTS].Video.Height))
        {
            if (Preview)
            {
                fprintf(File, "video = BilinearResize(video, %d, %d, %d, %d, %d, %d)\n",
                              VideoResWUpDown->Position, VideoResHUpDown->Position,
                              CropLeft, CropTop, CropWidth, CropHeight);

            }
            else
            {
                fprintf(File, "video = LanczosResize(video, %d, %d, %d, %d, %d, %d)\n",
                              VideoResWUpDown->Position, VideoResHUpDown->Position,
                              CropLeft, CropTop, CropWidth, CropHeight);
            }
        }
        else
        {
            if (Preview)
                fprintf(File, "video = BilinearResize(video, %d, %d)\n", VideoResWUpDown->Position, VideoResHUpDown->Position);
            else
                fprintf(File, "video = LanczosResize(video, %d, %d)\n", VideoResWUpDown->Position, VideoResHUpDown->Position);
        }

        // Subpicture
        if (IsSubpic && SubCheck->Checked)
        {
            // Rename ".sub" file if required
            NewSubFilename.printf("%s_st%d.sub", ReIndexName.c_str(), TitleInfo[VTS].PGC[PGC].Subpic[SubLangCombo->ItemIndex].Index);

            if (TitleInfo[VTS].SubFilename[TitleInfo[VTS].PGC[PGC].Subpic[SubLangCombo->ItemIndex].Index] != NewSubFilename)
            {
                if (!RenameFile(JobPath + TitleInfo[VTS].SubFilename[TitleInfo[VTS].PGC[PGC].Subpic[SubLangCombo->ItemIndex].Index], JobPath + NewSubFilename))
                {
                    // Close file
                    fclose(File);

                    return false;
                }
                TitleInfo[VTS].SubFilename[TitleInfo[VTS].PGC[PGC].Subpic[SubLangCombo->ItemIndex].Index] = NewSubFilename;
            }

            // Include into the script
            fprintf(File, "video = VobSub(video, \"%s_st%d.idx\")\n", ReIndexName.c_str(), TitleInfo[VTS].PGC[PGC].Subpic[SubLangCombo->ItemIndex].Index);
        }

        fprintf(File, "\n");

        // Audio
        if (IsAudio)
        {
            if (TitleInfo[VTS].PGC[PGC].Audio[Audio1TrackCombo->ItemIndex].Format == AF_LPCM)
            {
                fprintf(File, "audio = %s(\"%s_at%d.vfm\", samplerate=%d, samplebits=%d, channels=%d)",
                        FSKAudioCodecs[TitleInfo[VTS].PGC[PGC].Audio[Audio1TrackCombo->ItemIndex].Format].FuncName,
                        ReIndexName.c_str(), TitleInfo[VTS].PGC[PGC].Audio[Audio1TrackCombo->ItemIndex].Index,
                        TitleInfo[VTS].PGC[PGC].Audio[Audio1TrackCombo->ItemIndex].Samplerate,
                        TitleInfo[VTS].PGC[PGC].Audio[Audio1TrackCombo->ItemIndex].Quant,
                        TitleInfo[VTS].PGC[PGC].Audio[Audio1TrackCombo->ItemIndex].Channels);
            }
            else
            {
                int ch;

                if (Audio1MonoRadio->Checked)
                    ch = 1;
                else
                    if (Audio1StereoRadio->Checked || Audio1SurroundRadio->Checked)
                        ch = 2;
                    else
                        ch = 6;

                fprintf(File, "audio = %s(\"%s_at%d.vfm\", channels=%d)\n", FSKAudioCodecs[TitleInfo[VTS].PGC[PGC].Audio[Audio1TrackCombo->ItemIndex].Format].FuncName,
                                                                          ReIndexName.c_str(), TitleInfo[VTS].PGC[PGC].Audio[Audio1TrackCombo->ItemIndex].Index, ch);
            }

            // Gain
            if (Audio1GainCheck->Checked)
                fprintf(File, "audio = AmplifydB(audio, %f)\n", double(-Audio1GainTrack->Position) / 5);

            fprintf(File, "\n");
        }

        // Output
        if (IsAudio)
        {
            if (TitleInfo[VTS].PGC[PGC].Audio[Audio1TrackCombo->ItemIndex].Delay == 0)
                fprintf(File, "AudioDub(video, audio)\n");
            else
            {
                AudioDelay = float(TitleInfo[VTS].PGC[PGC].Audio[Audio1TrackCombo->ItemIndex].Delay) / 1000;
                fprintf(File, "AudioDub(video, DelayAudio(audio, %f))\n", AudioDelay);
            }
        }
        else
            fprintf(File, "video\n");

        // PAL TruSpeed
        if (VideoTruCheck->Checked)
            fprintf(File, "AssumeFPS(24, sync_audio=true)\n");

        // Trim
        if (VolumeCount > 1)
        {
            int First, Last;    // first and last frame

            First = (i - 1) * FramesPerVol;
            if (i == VolumeCount)
                Last = 0;
            else
                Last = (i * FramesPerVol) - 1;

            fprintf(File, "Trim(%d, %d)\n", First, Last);
        }

        // Verification
        if (!Preview)
            fprintf(File, "\nWriteFileStart(\""FSKJ_AVS_DEBUG_FILENAME"\", \"\"\" \"FilmShrink\" \"\"\")\n");

        // Close file
        fclose(File);
    }

    // Copy codecs
    if (!FileExists(JobPath + FSK_VIDEO_DECODER))
    {
        if (!CopyFile((FSKAppPath + FSK_VIDEO_DECODER).c_str(), (JobPath + FSK_VIDEO_DECODER).c_str(), FALSE))
            return false;
    }

    if ((IVTC || Deinterlace) && !FileExists(JobPath + FSK_VIDEO_DECOMB))
    {
        if (!CopyFile((FSKAppPath + FSK_VIDEO_DECOMB).c_str(), (JobPath + FSK_VIDEO_DECOMB).c_str(), FALSE))
            return false;
    }

    if (IsAudio && !FileExists(JobPath + AudioDec))
    {
        if (!CopyFile((FSKAppPath + AudioDec).c_str(), (JobPath + AudioDec).c_str(), FALSE))
            return false;
    }

    if (IsSubpic && SubCheck->Checked && !FileExists(JobPath + FSK_SUBPIC_DECODER))
    {
        if (!CopyFile((FSKAppPath + FSK_SUBPIC_DECODER).c_str(), (JobPath + FSK_SUBPIC_DECODER).c_str(), FALSE))
            return false;
    }

    // Success
    return true;
}

// Make the RealProducer job file(s)
bool TFSKNewForm::S4_MakeRPJF()
{
    int        VolumeCount;                             // number of volumes
    int        RollSize;                                // destination roll size
    AnsiString FileName, InFileName, OutFileName;       // filenames
    int        AvgBitrate, MaxBitrate, VideoQuality;    // stream params
    bool       Success;

    // Get number of volumes
    if (EncModeFixedRadio->Checked && VolCheck->Checked)
        VolumeCount = VolNumUpDown->Position;
    else
        VolumeCount = 1;

    // Create job files
    for (int i = 1; i <= VolumeCount; i++)
    {
        // Job filename
        FileName = JobPath;
        FileName.cat_printf(FSK_OUT_FILENAME, i);
        FileName.cat_printf(".rpjf");

        // AVS filename
        InFileName.printf(FSK_OUT_FILENAME, i);
        InFileName.cat_printf(".avs");

        // Output filename
        OutFileName.printf("..\\");
        OutFileName = OutFileName + JobName;
        if (VolumeCount > 1)
            OutFileName.cat_printf(" (%dof%d).rmvb", i, VolumeCount);
        else
            OutFileName.cat_printf(".rmvb");

        // Write file
        if (VolCheck->Checked && !EncModeFixedRadio->Checked)
            RollSize = VolUpDown->Position;
        else
            RollSize = 0;

        if (EncModeQualityRadio->Checked)
        {
            MaxBitrate   = 100000000;         // emulate vbrUnconstrainedQuality
            AvgBitrate   = 100000000;
            VideoQuality = EncModeQualityUpDown->Position;
        }
        else
        {
            AvgBitrate   = int(Bitrate);
            MaxBitrate   = AvgBitrate * 2;
            VideoQuality = -1;
        }

        if (IsAudio)
            Success = FSKWriteRPJF(FileName.c_str(),
                                   InfoTitleEdit->Text.c_str(), InfoAuthorEdit->Text.c_str(), InfoCopyEdit->Text.c_str(), InfoWordsEdit->Text.c_str(),
                                   InfoDescMemo->Text.c_str(), InfoRatingCombo->ItemIndex,
                                   InFileName.c_str(), OutFileName.c_str(),
                                   EHQTwoPassCheck->Checked,
                                   RollSize,
                                   AvgBitrate, MaxBitrate,
                                   EHQ2UpDown->Position, EHQ1UpDown->Position,
                                   enableRPR,
                                   rcEnableCurveCompression,
                                   rcHighBitrateReduce, rcLowBitrateBoost,
                                   rcPFrameRefQuant, rcBFrameRefQuant,
                                   AdaptiveB,
                                   maxConsecutiveBFrames, patternAdaptivity,
                                   inloopCutOffQuant,
                                   inloopCutOffCompatible, inloopCutOffBUseRefQuant,
                                   chromaModeDecision, encodeAllFrames,
                                   VideoQuality,
                                   Framerate,
                                   MiscLossCheck->Checked,

                                   true,
                                   (char *)AudioPresets[Audio1BitrateTrack->Position].Type, AudioPresets[Audio1BitrateTrack->Position].Flavor);
        else
            Success = FSKWriteRPJF(FileName.c_str(),
                                   InfoTitleEdit->Text.c_str(), InfoAuthorEdit->Text.c_str(), InfoCopyEdit->Text.c_str(), InfoWordsEdit->Text.c_str(),
                                   InfoDescMemo->Text.c_str(), InfoRatingCombo->ItemIndex,
                                   InFileName.c_str(), OutFileName.c_str(),
                                   EHQTwoPassCheck->Checked,
                                   RollSize,
                                   AvgBitrate, MaxBitrate,
                                   EHQ2UpDown->Position, EHQ1UpDown->Position,
                                   enableRPR,
                                   rcEnableCurveCompression,
                                   rcHighBitrateReduce, rcLowBitrateBoost,
                                   rcPFrameRefQuant, rcBFrameRefQuant,
                                   AdaptiveB,
                                   maxConsecutiveBFrames, patternAdaptivity,
                                   inloopCutOffQuant,
                                   inloopCutOffCompatible, inloopCutOffBUseRefQuant,
                                   chromaModeDecision, encodeAllFrames,
                                   VideoQuality,
                                   Framerate,
                                   MiscLossCheck->Checked,

                                   false);

        // Check if it was successful
        if (!Success)
            return false;
    }

    // Success
    return true;
}


void __fastcall TFSKNewForm::Audio1GainCheckClick(TObject *Sender)
{
    if (Audio1GainCheck->Checked)
    {
        Audio1GainTrack->Enabled  = true;
        Audio1GainLabel1->Enabled = true;
        Audio1GainLabel2->Enabled = true;
        Audio1GainLabel3->Enabled = true;
        Audio1GainLabel4->Enabled = true;
        Audio1GainLabel5->Enabled = true;
    }
    else
    {
        Audio1GainTrack->Enabled  = false;
        Audio1GainLabel1->Enabled = false;
        Audio1GainLabel2->Enabled = false;
        Audio1GainLabel3->Enabled = false;
        Audio1GainLabel4->Enabled = false;
        Audio1GainLabel5->Enabled = false;
    }
}

