/*=============================================================================
    Project Name:
            FilmShrink

	Module Name:
			FSKNew.h

	Abstract:
			FilmShrink "New Job" wizard

    License:
            GNU General Public License (GPL)

	Revision History:
			* Aug  1, 2004:  Created by Attila Afra
            * Jan  5, 2005:  Subpicture support
            * Jan 30, 2005:  Audio and subpicture detection fixed
            * Feb  5, 2005:  Major changes
            * Feb  6, 2005:  Bitrate calculation bug fixed

------------------------------------------------------------------------------
	Copyright  2004-2005 Attila T. Afra. All rights reserved.
=============================================================================*/

#ifndef FSKNewH
#define FSKNewH

// Includes
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include <ExtCtrls.hpp>
#include <Graphics.hpp>
#include <ComCtrls.hpp>
#include <ImgList.hpp>

#include "MainMacros.h"
#include "m2index.h"
#include "vtsifo.h"
#include "FSKTemplate.h"
#include <jpeg.hpp>

/*-----------------------------------------------------------------------------
    TFSKNewForm class
-----------------------------------------------------------------------------*/

class TFSKNewForm : public TForm
{
__published:
    // IDE-managed components
    TPanel *Header;
    TImage *HeaderBack;
    TImage *HeaderIcon;
    TImageList *HeaderIconList;
    TLabel *HeaderTitle;
    TLabel *HeaderComment;
    TBevel *LineBevel;
    TButton *NextButton;
    TButton *BackButton;
    TButton *CancelButton;
    TNotebook *Notebook;

    //  Step 1
    TLabeledEdit *NameEdit;
    TLabeledEdit *LocEdit;
    TLabeledEdit *SrcEdit;
    TButton *LocBrowseButton;
    TButton *SrcBrowseButton;
    TImage *PathIcon;
    TLabel *PathLabel;

    //  Step 2
    TListView *LogList;
    TProgressBar *ProgressBar;
    TLabel *TimeTextLabel;
    TImageList *LogIconList;
    TLabel *TimeLabel;

    //  Step 3
    TTreeView *TitleTree;
    TImageList *TreeIconList;
    TImage *PreviewImage;
    TImage *NoTitleImage;
    TBevel *PreviewBevel;
    TTrackBar *PreviewBar;
    TImageList *StreamIconList;
    TPageControl *StreamPageControl;
    TTabSheet *VideoSheet;
    TTabSheet *AudioSheet;
    TTabSheet *SubSheet;
    TImageList *AudioStreamIconList;
    TImageList *VolumeIconList;
    TLabel *AudioLabel1;
    TLabel *AudioLabel2;
    TListView *VideoInfoList;
    TLabel *AudioFormatLabel;
    TLabel *AudioModeLabel;
    TCheckBox *SubCheck;
    TLabel *SubLabel1;
    TComboBoxEx *SubLangCombo;
    TImage *RealLogo;
    TTabSheet *GeneralSheet;
    TGroupBox *EncModeGroupBox;
    TLabel *EncModeBitrateLabel;
    TLabel *EncModeQualityLabel;
    TRadioButton *EncModeFixedRadio;
    TRadioButton *EncModeBitrateRadio;
    TEdit *EncModeBitrateEdit;
    TUpDown *EncModeBitrateUpDown;
    TRadioButton *EncModeQualityRadio;
    TEdit *EncModeQualityEdit;
    TUpDown *EncModeQualityUpDown;
    TGroupBox *VideoResizeGroupBox;
    TImage *VideoResImage;
    TGroupBox *VideoCropGroupBox;
    TImage *VideoCropImage;
    TEdit *VideoCropBEdit;
    TEdit *VideoCropTEdit;
    TEdit *VideoCropREdit;
    TEdit *VideoCropLEdit;
    TRadioButton *VideoResAutoCheck;
    TRadioButton *VideoCropAutoCheck;
    TRadioButton *VideoResManuCheck;
    TRadioButton *VideoCropManuCheck;
    TRadioButton *VideoCropNoCheck;
    TButton *VideoCropAdjButton;
    TEdit *VideoResWEdit;
    TEdit *VideoResHEdit;
    TUpDown *VideoResHUpDown;
    TUpDown *VideoResWUpDown;
    TUpDown *VideoCropLUpDown;
    TUpDown *VideoCropRUpDown;
    TUpDown *VideoCropTUpDown;
    TUpDown *VideoCropBUpDown;
    TButton *PreviewButton;
    TGroupBox *VolGroupBox;
    TCheckBox *VolCheck;
    TComboBoxEx *VolCombo;
    TEdit *VolEdit;
    TUpDown *VolUpDown;
    TLabel *VolLabel;
    TEdit *VolNumEdit;
    TUpDown *VolNumUpDown;
    TLabel *VolNumLabel;
    TLabel *VideoInfoLabel;
    TGroupBox *EHQGroupBox;
    TTrackBar *EHQTrack;
    TGroupBox *VideoAdvGroupBox;
    TCheckBox *VideoTruCheck;
    TEdit *EHQ2Edit;
    TUpDown *EHQ1UpDown;
    TEdit *EHQ1Edit;
    TUpDown *EHQ2UpDown;
    TCheckBox *EHQCustomCheck;
    TCheckBox *EHQTwoPassCheck;
    TLabel *EHQ1Label2;
    TLabel *EHQ2Label2;
    TLabel *EHQ1Label1;
    TLabel *EHQ2Label1;
    TGroupBox *MiscGroupBox;
    TCheckBox *MiscLossCheck;
    TButton *VideoFiltersButton;
    TButton *VideoCodecButton;
    TGroupBox *Audio1GainGroupBox;
    TTrackBar *Audio1GainTrack;
    TCheckBox *Audio1GainCheck;
    TComboBoxEx *Audio1TrackCombo;
    TLabel *Audio1TrackLabel;
    TLabel *Audio1GainLabel2;
    TLabel *Audio1GainLabel3;
    TLabel *Audio1GainLabel1;
    TLabel *Audio1GainLabel4;
    TLabel *Audio1GainLabel5;
    TGroupBox *Audio1EncModeGroupBox;
    TRadioButton *Audio1MonoRadio;
    TRadioButton *Audio1StereoRadio;
    TRadioButton *Audio1SurroundRadio;
    TGroupBox *Audio1BitrateGroupBox;
    TTrackBar *Audio1BitrateTrack;
    TLabel *Audio1BitrateLabel;
    TRadioButton *Audio1BitrateAutoRadio;
    TRadioButton *Audio1BitrateManuRadio;
    TRadioButton *Audio1MultiRadio;
    TTabSheet *InfoSheet;
    TEdit *InfoTitleEdit;
    TEdit *InfoAuthorEdit;
    TEdit *InfoCopyEdit;
    TEdit *InfoWordsEdit;
    TMemo *InfoDescMemo;
    TComboBox *InfoRatingCombo;
    TLabel *InfoTitleLabel;
    TLabel *InfoAuthorLabel;
    TLabel *InfoCopyLabel;
    TLabel *InfoWordsLabel;
    TLabel *InfoDescLabel;
    TLabel *RatingLabel;
    TPanel *TitleLabel;

    // Events
    void __fastcall FormClose(TObject *Sender, TCloseAction &Action);

    //  Step 1
    void __fastcall NameEditChange(TObject *Sender);
    void __fastcall LocEditChange(TObject *Sender);
    void __fastcall SrcEditChange(TObject *Sender);
    void __fastcall LocBrowseButtonClick(TObject *Sender);
    void __fastcall SrcBrowseButtonClick(TObject *Sender);
    void __fastcall NextButtonClick(TObject *Sender);
    void __fastcall CancelButtonClick(TObject *Sender);
    void __fastcall FormShow(TObject *Sender);
    void __fastcall BackButtonClick(TObject *Sender);

    //  Step 3
    void __fastcall TitleTreeChange(TObject *Sender, TTreeNode *Node);
    void __fastcall PreviewBarChange(TObject *Sender);
    void __fastcall TitleTreeKeyPress(TObject *Sender, char &Key);

    //  Step 4
    void __fastcall VideoResEditKeyPress(TObject *Sender, char &Key);
    void __fastcall VideoResWEditChange(TObject *Sender);
    void __fastcall EncModeFixedRadioClick(TObject *Sender);
    void __fastcall EncModeBitrateRadioClick(TObject *Sender);
    void __fastcall EncModeQualityRadioClick(TObject *Sender);
    void __fastcall VolCheckClick(TObject *Sender);
    void __fastcall NumEditKeyPress(TObject *Sender, char &Key);
    void __fastcall EncModeBitrateEditChange(TObject *Sender);
    void __fastcall EncModeQualityEditChange(TObject *Sender);
    void __fastcall VolEditChange(TObject *Sender);
    void __fastcall VolNumEditChange(TObject *Sender);
    void __fastcall EncModeBitrateEditExit(TObject *Sender);
    void __fastcall EncModeQualityEditExit(TObject *Sender);
    void __fastcall VolEditExit(TObject *Sender);
    void __fastcall VolComboChange(TObject *Sender);
    void __fastcall VideoResHEditChange(TObject *Sender);
    void __fastcall VideoResHEditExit(TObject *Sender);
    void __fastcall Audio1TrackComboChange(TObject *Sender);
    void __fastcall Audio1BitrateTrackChange(TObject *Sender);
    void __fastcall Audio1BitrateRadioClick(TObject *Sender);
    void __fastcall PreviewButtonClick(TObject *Sender);
    void __fastcall VideoTruCheckClick(TObject *Sender);
    void __fastcall EHQCustomCheckClick(TObject *Sender);
    void __fastcall EHQTwoPassCheckClick(TObject *Sender);
    void __fastcall EHQTrackChange(TObject *Sender);
    void __fastcall SubCheckClick(TObject *Sender);
    void __fastcall FormActivate(TObject *Sender);
    void __fastcall VideoResWEditExit(TObject *Sender);
    void __fastcall VideoResAutoCheckClick(TObject *Sender);
    void __fastcall VideoResManuCheckClick(TObject *Sender);
    void __fastcall VideoCropTEditExit(TObject *Sender);
    void __fastcall VideoCropBEditExit(TObject *Sender);
    void __fastcall VideoCropLEditExit(TObject *Sender);
    void __fastcall VideoCropREditExit(TObject *Sender);
    void __fastcall VideoCropNoCheckClick(TObject *Sender);
    void __fastcall VideoCropAutoCheckClick(TObject *Sender);
    void __fastcall VideoCropManuCheckClick(TObject *Sender);
    void __fastcall VideoCropTEditChange(TObject *Sender);
    void __fastcall VideoCropBEditChange(TObject *Sender);
    void __fastcall VideoCropLEditChange(TObject *Sender);
    void __fastcall VideoCropREditChange(TObject *Sender);
    void __fastcall VideoCropAdjButtonClick(TObject *Sender);
    void __fastcall VideoFiltersButtonClick(TObject *Sender);
    void __fastcall VideoCodecButtonClick(TObject *Sender);
    void __fastcall Audio1EncModeRadioClick(TObject *Sender);
    void __fastcall Audio1GainCheckClick(TObject *Sender);
    void __fastcall EHQ1EditExit(TObject *Sender);
    void __fastcall EHQ2EditExit(TObject *Sender);
    void __fastcall NameEditExit(TObject *Sender);
    void __fastcall NameEditKeyPress(TObject *Sender, char &Key);
    void __fastcall InfoEditKeyPress(TObject *Sender, char &Key);
    void __fastcall InfoTitleEditExit(TObject *Sender);
    void __fastcall InfoAuthorEditExit(TObject *Sender);
    void __fastcall InfoCopyEditExit(TObject *Sender);
    void __fastcall InfoWordsEditExit(TObject *Sender);
    void __fastcall InfoDescMemoExit(TObject *Sender);

// User declarations
private:
    MAIN_WNDPROC_PROTO    // event handler

    // Friends
    friend class TFSKAnalyzeThread;
    friend class TFSKSelectThread;
    friend class TFSKNewFiltersForm;
    friend class TFSKNewCodecForm;
    friend class TFSKNewCropForm;

    // Shared
    AnsiString JobName, JobRoot, JobPath,                         // job paths
               DVDPath, OutPath, PreviewPath;                     // other paths

    int        VTS, PGC, Angle;                                   // selected title
    double     VideoAR;                                           // video aspect ratio
    int        NumFrames, OriNumFrames, _NumFrames;               // number of frames
    float      Framerate, OriFramerate;                           // framerate (& original)
    double     NumSeconds, OriNumSeconds;                         // number of seconds (& original)
    int        CropTop, CropLeft, CropWidth, CropHeight;          // crop values
    bool       IsAudio, IsSubpic;                                 // do audio/subpicture tracks exists?
    AnsiString ReIndexName;                                       // rebuilt index root filename
    const      TFSKAudioPreset *AudioPresets, *PrevAudioPresets;  // audio preset array
    const      TFSKVideoComplexity *VideoComplexity;              // video complexity presets
    int        AudioPresetCount;                                  // number of presets in the audio preset array
    double     Bitrate, VideoBitrate, AudioBitrate,               // total/video/audio bitrate
               OriBitrate, OriVideoBitrate;                       // total/video/audio bitrate (at original framerate/duration)
    bool       Deinterlace;                                       // deinterlace flag
    bool       FullDeinterlace;                                   // full deinterlace method flag
    bool       IVTC;                                              // inverse telecine flag
    bool       ComplexIVTC;                                       // complex inverse telecine method flag
    bool       ComplexIVTCDecimate;                               // IVTC decimation flag (complex method only)
    bool       IsTFF;                                             // top field first flag

    // Codec options
    bool       ManualVideoCodec;
    bool       AdaptiveB;

    bool       rcEnableCurveCompression;
    int        rcHighBitrateReduce, rcLowBitrateBoost;
    int        rcPFrameRefQuant, rcBFrameRefQuant;
    int        inloopCutOffQuant;
    bool       inloopCutOffCompatible, inloopCutOffBUseRefQuant;
    int        maxConsecutiveBFrames, patternAdaptivity;
    bool       encodeAllFrames;
    bool       chromaModeDecision;
    bool       enableRPR;

    //  Audio info structure
    struct TAudioInfo
    {
        char           Index;       // index of track
        m2AudioFormat  Format;      // audio format
        char           Channels;    // number of channels
        short          Delay;       // delay in milliseconds
        unsigned short Language;    // language code
        char           Caption;     // 0 - normal; 3 - comment

        // LPCM only
        char           Quant;       // sample precision
        int            Samplerate;  // sample frequency
    };

    //  Subpicture info structure
    struct TSubpicInfo
    {
        char           Index;        // index of track
        unsigned short Language;     // language code
    };

    //  PGC info structure
    struct TPGCInfo
    {
        char       AngleNum;  // number of angles
        dvd_time_t Length;    // length of PGC

        // Video (cropped)
        struct
        {
            int  NumFrames;                 // number of frames
            char Crop;                      // 0 = no crop; 1 = crop; -1 = unknown
            int  Top, Left, Width, Height;
        } Video;

        // Audio
        char         AudioNum;    // number of audio tracks
        TAudioInfo  *Audio;       // audio info table

        // Subpicture
        char         SubpicNum;   // number of subpicture tracks
        TSubpicInfo *Subpic;      // subpicture info table
    };

    //  Title info table
    struct
    {
        // Video
        struct
        {
            short Width, Height;    // frame dimensions
            bool  IsWide;           // is the aspect ratio 16:9?
            bool  IsPAL;            // is the TV system PAL?
            float Framerate;        // framerate

            bool  IsProgressive;    // are the frames mostly progressive?
            float FrameTypePurity;  // frame type purity in %
            bool  IsTFF;            // is top field first (field order)?
            float FilmPurity;       // Film purity in %
        } Video;

        // PGC
        char      PGCNum;    // number of PGCs
        TPGCInfo *PGC;       // PGC info table

        // Subpicture
        AnsiString SubFilename[IFO_NUM_SPU];  // current filename of the subfile (array)
    } TitleInfo[100];

    // Step 1
    bool S1_IsFirstTime;                    // is it the first entering this step ?
    AnsiString S1_SrcPath, S1_JobLoc;       // paths
    int S1_PrevPathLen, S1_PrevSrcPathLen;
    bool S1_RootSrcPath;                    // does SrcPath point to the root of the DVD?

    void S1_Begin();
    bool S1_End(bool Forward);

    bool S1_UpdatePath();
    void S1_HidePath();
    bool S1_VerifySrcPath();
    bool S1_VerifyDVD(AnsiString Path);
    bool S1_VerifyDVD();

    // Step 2
    bool S2_Finished;               // is the analysis finished?
    bool S2_Active;                 // is the thread active?
    bool S2_AbortFlag;              // abort was requested
    TFSKAnalyzeThread *AnalyzeThread;  // this thread does the analyzation process

    enum ELogStateIcon
    {
        LSI_Info,
        LSI_Warning,
        LSI_Error,
        LSI_Video,
    };

    void S2_Begin();
    bool S2_End(bool Forward);

    void S2_Finish(bool Success);
    bool S2_Abort(bool Back);

    void AddLogEvent(ELogStateIcon StateIcon, AnsiString Event);

    // Step 3
    bool               S3_IsFirstEndCall;                      // is it the first end call since begin?
    bool               S3_Reset;                               // do we have to reset the stuff when entering this pass?
    bool               S3_Selected;                            // valid tree selection?
    AnsiString         S3_FileName;                            // filename of the preview index
    int                S3_SelWidth, S3_SelHeight;              // selected title frame width/height
    int                S3_Pre43Top, S3_Pre43Height;            // preview image top/height in 4:3 mode
    int                S3_Pre169Top, S3_Pre169Height;          // preview image top/height in 16:9 mode
    Graphics::TBitmap *S3_PreviewFrame;                        // preview frame data
    int                S3_PrevVTS, S3_PrevPGC, S3_PrevAngle;   // previously selected title
    bool               S3_PreviewActive;                       // is the preview function active?
    bool               S3_TitleTreeRebuildOnly;

    void S3_Begin();
    bool S3_End(bool Forward);

    // Step 4
    bool S4_Reset;                  // do we have to reset the stuff when entering this pass?
    bool S4_IsActive;               // are the events active?
    int  S4_MinOutputSize;          // minimum output size
    bool S4_DoneButtonEnabled;

    void S4_Begin();
    bool S4_End(bool Forward);

    void S4_SetDefault();           // set default settings
    void S4_CalcVideoAR();          // calculate video aspect ratio
    double S4_CalcVideoAR(bool Crop, bool Auto);
    void S4_AdjustResolution(int& Width, int& Height, double AR, bool WRef);
    double S4_FramesToSeconds(int Frames, float Framerate);
    void S4_UpdateDuration();
    void S4_UpdateDecimate();
    void S4_UpdateBitrate(bool Full = true, bool UpdateAudioMode = false);
    void S4_UpdateAutoResize();
    void S4_UpdateCrop();
    void S4_UpdateVideoCodec();
    void S4_UpdateAudioMode();
    void S4_UpdateAudioBitrate(bool Full);
    void S4_UpdateVideoComplexity(const TFSKVideoComplexity *NewVideoComplexity);

    bool S4_MakeAVS(bool Preview);
    bool S4_MakeRPJF();

public:
    // Constructors
    __fastcall TFSKNewForm(TComponent* Owner);
    __fastcall ~TFSKNewForm();
};

// External FSKNewForm declaration
extern PACKAGE TFSKNewForm *FSKNewForm;

#endif  // NewH
