/*=============================================================================
    Project Name:
            FilmShrink

	Module Name:
			FSKNewCrop.cpp

	Abstract:
			FilmShrink "New Job" wizard's video crop form

    License:
            GNU General Public License (GPL)

	Revision History:
			* Mar  7, 2005:  Created by Attila Afra
            * Mar 23, 2005:  Zoom function
            * Mar 29, 2005:  Improved TEdit->Text verification

------------------------------------------------------------------------------
	Copyright  2005 Attila T. Afra. All rights reserved.
=============================================================================*/

// Warnings
#pragma warn -8060  // Possibly incorrect assignment

#include <vcl.h>
#pragma hdrstop

// Includes
#include "m2preview.h"

#include "FSKGlobal.h"
#include "FSKNew.h"
#include "FSKNewCrop.h"

// VCL stuff
#pragma package(smart_init)
#pragma resource "*.dfm"

/*=============================================================================
    TFSKNewCropForm
=============================================================================*/

TFSKNewCropForm *FSKNewCropForm;

/*-----------------------------------------------------------------------------
    TFSKNewCropForm >> Constructors
-----------------------------------------------------------------------------*/

__fastcall TFSKNewCropForm::TFSKNewCropForm(TComponent* Owner) : TForm(Owner)
{
    // Initialize image transparency
    VideoCropImage->Picture->Bitmap->TransparentColor = clFuchsia;
}

/*-----------------------------------------------------------------------------
    TFSKNewCropForm >> Events
-----------------------------------------------------------------------------*/

void __fastcall TFSKNewCropForm::FormShow(TObject *Sender)
{
    // Disable parent form
    FSKNewForm->Enabled = false;

    // Get crop info
    Update = PreviewActive = EnableCropMode = false;
    CropTop = CropLeft = CropWidth = CropHeight = 0;

    VideoCropTUpDown->Position = FSKNewForm->VideoCropTUpDown->Position;
    VideoCropBUpDown->Position = FSKNewForm->VideoCropBUpDown->Position;
    VideoCropLUpDown->Position = FSKNewForm->VideoCropLUpDown->Position;
    VideoCropRUpDown->Position = FSKNewForm->VideoCropRUpDown->Position;

    VideoCropTEditChange(Sender);
    VideoCropBEditChange(Sender);
    VideoCropLEditChange(Sender);
    VideoCropREditChange(Sender);
}

void __fastcall TFSKNewCropForm::FormClose(TObject *Sender, TCloseAction &Action)
{
    if (Update)
    {
        // Update crop info
        FSKNewForm->VideoCropTUpDown->Max = VideoCropTUpDown->Position;
        FSKNewForm->VideoCropBUpDown->Max = VideoCropBUpDown->Position;
        FSKNewForm->VideoCropLUpDown->Max = VideoCropLUpDown->Position;
        FSKNewForm->VideoCropRUpDown->Max = VideoCropRUpDown->Position;

        FSKNewForm->VideoCropTUpDown->Position = VideoCropTUpDown->Position;
        FSKNewForm->VideoCropBUpDown->Position = VideoCropBUpDown->Position;
        FSKNewForm->VideoCropLUpDown->Position = VideoCropLUpDown->Position;
        FSKNewForm->VideoCropRUpDown->Position = VideoCropRUpDown->Position;
    }

    // Free memory
    m2PreviewClose();
    delete Frame;

    // Enable parent form
    FSKNewForm->Enabled = true;
}

/*---------------------------------------------------------------------------*/

void __fastcall TFSKNewCropForm::VideoCropTEditChange(TObject *Sender)
{
    if (!VideoCropTEdit->Text.IsEmpty())
    {
        try
        {
            VideoCropTEdit->Text.ToInt();
        }
        catch (...)
        {
            return;
        }

        CropHeight += CropTop;
        CropTop     = VideoCropTUpDown->Position;
        CropHeight -= CropTop;

        VideoCropTUpDown->Max = FSKNewForm->TitleInfo[FSKNewForm->VTS].Video.Height - VideoCropBUpDown->Position - FSK_MIN_VIDEO_RES;
        VideoCropBUpDown->Max = FSKNewForm->TitleInfo[FSKNewForm->VTS].Video.Height - VideoCropTUpDown->Position - FSK_MIN_VIDEO_RES;
        VideoCropTUpDown->Refresh();
        VideoCropBUpDown->Refresh();
    }

    UpdateFrame();
}

void __fastcall TFSKNewCropForm::VideoCropBEditChange(TObject *Sender)
{
    if (!VideoCropBEdit->Text.IsEmpty())
    {
        try
        {
            VideoCropBEdit->Text.ToInt();
        }
        catch (...)
        {
            return;
        }

        CropHeight = FSKNewForm->TitleInfo[FSKNewForm->VTS].Video.Height - CropTop - VideoCropBUpDown->Position;

        VideoCropTUpDown->Max = FSKNewForm->TitleInfo[FSKNewForm->VTS].Video.Height - VideoCropBUpDown->Position - FSK_MIN_VIDEO_RES;
        VideoCropBUpDown->Max = FSKNewForm->TitleInfo[FSKNewForm->VTS].Video.Height - VideoCropTUpDown->Position - FSK_MIN_VIDEO_RES;
        VideoCropTUpDown->Refresh();
        VideoCropBUpDown->Refresh();
    }

    UpdateFrame();
}

void __fastcall TFSKNewCropForm::VideoCropLEditChange(TObject *Sender)
{
    if (!VideoCropLEdit->Text.IsEmpty())
    {
        try
        {
            VideoCropLEdit->Text.ToInt();
        }
        catch (...)
        {
            return;
        }

        CropWidth += CropLeft;
        CropLeft   = VideoCropLUpDown->Position;
        CropWidth -= CropLeft;

        VideoCropLUpDown->Max = FSKNewForm->TitleInfo[FSKNewForm->VTS].Video.Width - VideoCropRUpDown->Position - FSK_MIN_VIDEO_RES;
        VideoCropRUpDown->Max = FSKNewForm->TitleInfo[FSKNewForm->VTS].Video.Width - VideoCropLUpDown->Position - FSK_MIN_VIDEO_RES;
        VideoCropLUpDown->Refresh();
        VideoCropRUpDown->Refresh();
    }

    UpdateFrame();
}

void __fastcall TFSKNewCropForm::VideoCropREditChange(TObject *Sender)
{
    if (!VideoCropREdit->Text.IsEmpty())
    {
        try
        {
            VideoCropREdit->Text.ToInt();
        }
        catch (...)
        {
            return;
        }

        CropWidth = FSKNewForm->TitleInfo[FSKNewForm->VTS].Video.Width - CropLeft - VideoCropRUpDown->Position;

        VideoCropLUpDown->Max = FSKNewForm->TitleInfo[FSKNewForm->VTS].Video.Width - VideoCropRUpDown->Position - FSK_MIN_VIDEO_RES;
        VideoCropRUpDown->Max = FSKNewForm->TitleInfo[FSKNewForm->VTS].Video.Width - VideoCropLUpDown->Position - FSK_MIN_VIDEO_RES;
        VideoCropLUpDown->Refresh();
        VideoCropRUpDown->Refresh();
    }

    UpdateFrame();
}

void __fastcall TFSKNewCropForm::VideoCropTEditExit(TObject *Sender)
{
    if (VideoCropTEdit->Text.IsEmpty())
    {
        Application->MessageBoxA("Please enter an integer.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
        VideoCropTEdit->SetFocus();
        return;
    }
    else
    {
        try
        {
            VideoCropTEdit->Text.ToInt();
        }
        catch (...)
        {
            Application->MessageBoxA("Please enter a valid integer.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
            VideoCropTEdit->SetFocus();
            return;
        }

        VideoCropTEdit->Text = (VideoCropTUpDown->Position / 2) * 2;
    }
}

void __fastcall TFSKNewCropForm::VideoCropBEditExit(TObject *Sender)
{
    if (VideoCropBEdit->Text.IsEmpty())
    {
        Application->MessageBoxA("Please enter an integer.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
        VideoCropBEdit->SetFocus();
        return;
    }
    else
    {
        try
        {
            VideoCropBEdit->Text.ToInt();
        }
        catch (...)
        {
            Application->MessageBoxA("Please enter a valid integer.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
            VideoCropBEdit->SetFocus();
            return;
        }

        VideoCropBEdit->Text = (VideoCropBUpDown->Position / 2) * 2;
    }
}

void __fastcall TFSKNewCropForm::VideoCropLEditExit(TObject *Sender)
{
    if (VideoCropLEdit->Text.IsEmpty())
    {
        Application->MessageBoxA("Please enter an integer.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
        VideoCropLEdit->SetFocus();
        return;
    }
    else
    {
        try
        {
            VideoCropLEdit->Text.ToInt();
        }
        catch (...)
        {
            Application->MessageBoxA("Please enter a valid integer.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
            VideoCropLEdit->SetFocus();
            return;
        }

        VideoCropLEdit->Text = (VideoCropLUpDown->Position / 2) * 2;
    }
}

void __fastcall TFSKNewCropForm::VideoCropREditExit(TObject *Sender)
{
    if (VideoCropREdit->Text.IsEmpty())
    {
        Application->MessageBoxA("Please enter an integer.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
        VideoCropREdit->SetFocus();
        return;
    }
    else
    {
        try
        {
            VideoCropREdit->Text.ToInt();
        }
        catch (...)
        {
            Application->MessageBoxA("Please enter a valid integer.", FSK_PRODUCT_NAME, MB_OK | MB_ICONEXCLAMATION);
            VideoCropREdit->SetFocus();
            return;
        }

        VideoCropREdit->Text = (VideoCropRUpDown->Position / 2) * 2;
    }
}

void __fastcall TFSKNewCropForm::VideoCropEditKeyPress(TObject *Sender, char &Key)
{
    // Ignore nondigit characters
    if (!isdigit(Key) && (Key != 8) /* Backspace */)
    {
        //Key = 27; /* Escape */
        Key = 0; // No key pressed
        MessageBeep(MB_OK);
    }
}

/*---------------------------------------------------------------------------*/

void __fastcall TFSKNewCropForm::OKButtonClick(TObject *Sender)
{
    Update = true;
    Close();
}

void __fastcall TFSKNewCropForm::CancelButtonClick(TObject *Sender)
{
    Close();
}

void __fastcall TFSKNewCropForm::ZoomButtonClick(TObject *Sender)
{
    int _Width, _Height;

    if (Half)
    {
        Half = false;
        ZoomButton->Caption = "Zoom Out";
        _Width = FullWidth;
        _Height = FullHeight;
    }
    else
    {
        Half = true;
        ZoomButton->Caption = "Zoom In";
        _Width = HalfWidth;
        _Height = HalfHeight;
    }

    if (Left + _Width > Screen->Width)
        Left = Screen->Width - _Width - 32;

    if (Top + _Height > Screen->Height)
        Top = Screen->Height - _Height - 32;

    Width = _Width;
    Height = _Height;

    TrackBarChange(Sender);
}

/*---------------------------------------------------------------------------*/

void __fastcall TFSKNewCropForm::TrackBarChange(TObject *Sender)
{
    int            i, j;         // counters
    int            FrameIdx;     // frame index
    unsigned char *Src, *Dest;   // source and destination pixel pointers

    if (PreviewActive)
        return;

    PreviewActive = true;

    // Get frame index
    FrameIdx = ((NumFrames - 1) * TrackBar->Position) / TrackBar->Max;

    // Get frame data
    if (!(Src = m2PreviewGet(FrameIdx)))
    {
        // Show error box
        Application->MessageBoxA("Error detected in the selected title! The DVD is probably damaged!", FSK_PRODUCT_NAME, MB_ICONERROR | MB_OK);

        // Close title
        m2PreviewClose();

        // Close window
        Close();
        return;
    }

    // Copy pixels to frame bitmap
    for (i = 0; i < FrameHeight; i++)
    {
        Dest = (unsigned char *)Frame->ScanLine[i];

        for (j = 0; j < FrameWidth; j++)
        {
            *Dest++ = *Src++;
            *Dest++ = *Src++;
            *Dest++ = *Src++;
        }
    }

    // Update frame
    UpdateFrame();

    PreviewActive = false;
}

/*---------------------------------------------------------------------------*/

void __fastcall TFSKNewCropForm::ImageMouseMove(TObject *Sender, TShiftState Shift, int X, int Y)
{
    if (Half)
    {
        X *= 2;
        Y *= 2;
    }

    // Update cursor
    if (EnableCropMode)
    {
        switch (CropMode)
        {
        case cmTop:
            VideoCropTUpDown->Position = (Y / 2) * 2;
            VideoCropTEdit->Refresh();
            break;

        case cmBottom:
            VideoCropBUpDown->Position = ((Frame->Height - Y - 1) / 2) * 2;
            VideoCropBEdit->Refresh();
            break;

        case cmLeft:
            VideoCropLUpDown->Position = (X / 2) * 2;
            VideoCropLEdit->Refresh();
            break;

        case cmRight:
            VideoCropRUpDown->Position = ((Frame->Width - X - 1) / 2) * 2;
            VideoCropREdit->Refresh();
            break;
        }
    }
    else
    {
        int Limit;

        if (Half)
            Limit = 6;
        else
            Limit = 3;

        if ((Y > CropTop - Limit) && (Y < CropTop + Limit))
        {
            CropMode = cmTop;
            Image->Cursor = crSizeNS;
        }
        else
            if ((Y > CropTop + CropHeight - Limit - 1) && (Y < CropTop + CropHeight + Limit - 1))
            {
                CropMode = cmBottom;
                Image->Cursor = crSizeNS;
            }
            else
                if ((X > CropLeft - Limit) && (X < CropLeft + Limit))
                {
                   CropMode = cmLeft;
                   Image->Cursor = crSizeWE;
                }
                else
                    if ((X > CropLeft + CropWidth - Limit - 1) && (X < CropLeft + CropWidth + Limit - 1))
                    {
                        CropMode = cmRight;
                        Image->Cursor = crSizeWE;
                    }
                    else
                    {
                        CropMode = cmNone;
                        Image->Cursor = crDefault;
                    }
    }
}

void __fastcall TFSKNewCropForm::ImageMouseDown(TObject *Sender, TMouseButton Button, TShiftState Shift, int X, int Y)
{
    EnableCropMode = CropMode != cmNone;

    // Update focus
    if (EnableCropMode)
    {
        switch (CropMode)
        {
        case cmTop:
            VideoCropTEdit->SetFocus();
            break;

        case cmBottom:
            VideoCropBEdit->SetFocus();
            break;

        case cmLeft:
            VideoCropLEdit->SetFocus();
            break;

        case cmRight:
            VideoCropREdit->SetFocus();
            break;
        }
    }
}

void __fastcall TFSKNewCropForm::ImageMouseUp(TObject *Sender, TMouseButton Button, TShiftState Shift, int X, int Y)
{
    EnableCropMode = false;
}

/*-----------------------------------------------------------------------------
    TFSKNewCropForm >> User methods
-----------------------------------------------------------------------------*/

// Initialization (call before Show()!)
bool TFSKNewCropForm::Init()
{
    Screen->Cursor = crHourGlass;

    if (m2PreviewOpen((FSKNewForm->JobPath + FSKNewForm->ReIndexName + ".m2vi").c_str(), FrameWidth, FrameHeight, NumFrames, int(FSKNewForm->TitleInfo[FSKNewForm->VTS].Video.FilmPurity) > 0))
    {
        Frame = new Graphics::TBitmap();
        Frame->PixelFormat = pf24bit;
        Frame->Width = FrameWidth;
        Frame->Height = FrameHeight;


        FullWidth = Width + FrameWidth - Image->Width;
        FullHeight = Height + FrameHeight - Image->Height;
        HalfWidth = Width + FrameWidth / 2 - Image->Width;
        HalfHeight = Height + FrameHeight / 2 - Image->Height;

        Half = true;
        Width = HalfWidth;
        Height = HalfHeight;

        TrackBarChange(NULL);
        Screen->Cursor = crDefault;
        return true;
    }
    else
    {
        Screen->Cursor = crDefault;
        return false;
    }
}

// Update frame
void TFSKNewCropForm::UpdateFrame()
{
    int i, j, x;

    Image->Picture->Bitmap = Frame;
    Image->Canvas->Pen->Color = clWhite;

    // Top
    Image->Picture->Bitmap->Canvas->MoveTo(0, CropTop);
    Image->Picture->Bitmap->Canvas->LineTo(FrameWidth, CropTop);

    // Bottom
    Image->Picture->Bitmap->Canvas->MoveTo(0, CropTop + CropHeight - 1);
    Image->Picture->Bitmap->Canvas->LineTo(FrameWidth, CropTop + CropHeight - 1);

    // Left
    Image->Picture->Bitmap->Canvas->MoveTo(CropLeft, 0);
    Image->Picture->Bitmap->Canvas->LineTo(CropLeft, FrameHeight);

    // Right
    Image->Picture->Bitmap->Canvas->MoveTo(CropLeft + CropWidth - 1, 0);
    Image->Picture->Bitmap->Canvas->LineTo(CropLeft + CropWidth - 1, FrameHeight);

    if (Half)
    {
        // Top
        Image->Picture->Bitmap->Canvas->MoveTo(0, CropTop + 1);
        Image->Picture->Bitmap->Canvas->LineTo(FrameWidth, CropTop + 1);

        // Bottom
        Image->Picture->Bitmap->Canvas->MoveTo(0, CropTop + CropHeight - 2);
        Image->Picture->Bitmap->Canvas->LineTo(FrameWidth, CropTop + CropHeight - 2);

        // Left
        Image->Picture->Bitmap->Canvas->MoveTo(CropLeft + 1, 0);
        Image->Picture->Bitmap->Canvas->LineTo(CropLeft + 1, FrameHeight);

        // Right
        Image->Picture->Bitmap->Canvas->MoveTo(CropLeft + CropWidth - 2, 0);
        Image->Picture->Bitmap->Canvas->LineTo(CropLeft + CropWidth - 2, FrameHeight);
    }

    // Refresh stuff
    Image->Refresh();
}



