/*=============================================================================
    Project Name:
            FilmShrink

	Module Name:
			FSKNewThreads.h

	Abstract:
			FilmShrink "New Job" threads

    License:
            GNU General Public License (GPL)

	Revision History:
			* Oct 13, 2004:  Created by Attila Afra
            * Feb 13, 2005:  Minor changes
            * Mar  5, 2005:  Index import support

------------------------------------------------------------------------------
	Copyright  2004-2005 Attila T. Afra. All rights reserved.
=============================================================================*/

#ifndef FSKNewThreadsH
#define FSKNewThreadsH

// Includes
#include "FSKNew.h"

/*-----------------------------------------------------------------------------
    TFSKAnalyzeThread class
-----------------------------------------------------------------------------*/

class TFSKAnalyzeThread : public TThread
{
public:
    // Constructor
    __fastcall TFSKAnalyzeThread(bool CreateSuspended);
    __fastcall ~TFSKAnalyzeThread();

    // Main methods
    void __fastcall Execute();
    void SuspendQuery();
    void ResumeQuery();

private:
    // External types
    typedef TFSKNewForm::ELogStateIcon ELogStateIcon;

    // Core
    bool Term;                             // thread termination flag
    bool Import;                           // import flag
    AnsiString JobPath, OutPath, DVDPath, ImportPath;
    __int64 TotalSize, CurSize, DoneSize;  // total/current/done stream size
    TDateTime StartTime;                   // date/time when the analyzis was started
    m2IndexInfo IdxInfo;                   // indexer info
    unsigned long DVDSum;                  // DVD checksum
    TStringList *IdxLog;                   // IdxLog file

    bool __inline CheckTerm();
    bool ACreateJob();
    bool ACheckImport();
    bool ATotalSize();
    bool AProcessStreams();
    bool AImportStreams();
    bool AProcessStream(AnsiString Name, int ID);

    // Form control methods
    void AddLogEvent(ELogStateIcon StateIcon, AnsiString Event);
    void Info(AnsiString Text);
    void Warning(AnsiString Text);
    void Error(AnsiString Text1, AnsiString Text2);
    void FatalError(AnsiString Text1, AnsiString Text2);
    void __inline Finish(bool Success);
    void __inline SetProgress(int Progress);
    void __inline SetTime(int Seconds);

    // Thread-safe methods
    ELogStateIcon _LogStateIcon;
    AnsiString _LogEvent;
    bool _Success;
    int _Progress;
    int _Time, _LastTime;

    void __fastcall _AddLogEvent();
    void __fastcall _Error();
    void __fastcall _Finish();
    void __fastcall _SetProgress();
    void __fastcall _SetTime();
};

/*-----------------------------------------------------------------------------
    TFSKSelectThread class
-----------------------------------------------------------------------------*/

class TFSKSelectThread : public TThread
{
public:
    // Constructors
    __fastcall TFSKSelectThread();

    // Main methods
    void __fastcall Execute();

    // Thread-safe methods
    void __fastcall _Finish();
    void __fastcall _Error();
};

#endif // FSKNewThreadsH
