/*=============================================================================
    Project Name:
            FilmShrink

	Module Name:
			FSKSplash.cpp

	Abstract:
			FilmShrink splash form

    License:
            GNU General Public License (GPL)

	Revision History:
			* Dec 24, 2004:  Created by Attila Afra
            * Jan 14, 2005:  About box
            * Jan 20, 2005:  OK button removed
            * Mar 23, 2005:  Menu form initialization check

------------------------------------------------------------------------------
	Copyright  2004-2005 Attila T. Afra. All rights reserved.
=============================================================================*/

#include <vcl.h>
#pragma hdrstop

// Includes
#include "FSKGlobal.h"
#include "FSKSplash.h"

// VCL stuff
#pragma package(smart_init)
#pragma resource "*.dfm"

/*=============================================================================
    TFSKSplashForm
=============================================================================*/

// SplashForm definition
TFSKSplashForm *FSKSplashForm;

/*-----------------------------------------------------------------------------
    TFSKSplashForm >> Constructors
-----------------------------------------------------------------------------*/

// Default constructor
__fastcall TFSKSplashForm::TFSKSplashForm(TComponent* Owner)
    : TForm(Owner)
{
    FirstShow = true;
    InitError = false;

    // Get version info
    DWORD      n = GetFileVersionInfoSize(Application->ExeName.c_str(), &n);
    char      *pBuf = new char[n];
    char      *pValue;
    DWORD      Len;
    AnsiString Comments;

    if (n > 0)
    {
        GetFileVersionInfo(Application->ExeName.c_str(), 0, n, pBuf);

        // Get comments
        if (VerQueryValue(pBuf, "StringFileInfo\\040904E4\\Comments", (void * *)&pValue, (unsigned int *)&Len))
            Comments = pValue;
        else
            goto Error;

        // Get file version
        if (VerQueryValue(pBuf, "StringFileInfo\\040904E4\\FileVersion", (void * *)&pValue, (unsigned int *)&Len))
            VersionLabel->Caption = AnsiString("Version ") + pValue + " " + Comments;
        else
            goto Error;

        // Get copyright
        if (VerQueryValue(pBuf, "StringFileInfo\\040904E4\\LegalCopyright", (void * *)&pValue, (unsigned int *)&Len))
            CopyrightLabel->Caption = pValue;
        else
            goto Error;
    }
    else
        goto Error;

    return;

    // Error
Error:
    delete pBuf;

    InitError = true;
    Application->MessageBoxA("Unable to retrieve version information!", FSK_PRODUCT_NAME, MB_OK | MB_ICONERROR);
    Application->Terminate();
}

/*-----------------------------------------------------------------------------
    TFSKSplashForm >> Events
-----------------------------------------------------------------------------*/

void __fastcall TFSKSplashForm::FormShow(TObject *Sender)
{
    AlphaBlendValue = 0;

    if (FirstShow)
        Cursor = crDefault;
    else
        Cursor = crHandPoint;

    FirstShow = false;
}

void __fastcall TFSKSplashForm::ImageClick(TObject *Sender)
{
    if (FirstShow)
        return;

    // Fade out
    Repaint();
    for (int i = 255; i >= 0; i -= 4)
        AlphaBlendValue = i;
    AlphaBlendValue = 0;
    Repaint();

    ParentForm->Enabled = true;
    Hide();
}

/*-----------------------------------------------------------------------------
    TFSKSplashForm >> Operations
-----------------------------------------------------------------------------*/

// FilmShrink main startup method
void TFSKSplashForm::FSKStartup()
{
    if (InitError)
        return;

    Screen->Cursor = crHourGlass;

    // Fade in
    Repaint();
    for (int i = 1; i < 256; i += 4)
        AlphaBlendValue = i;
    AlphaBlendValue = 255;
    Repaint();

    // Initialize FilmShrink
    if (!FSKInitialize())
    {
        Application->Terminate();
        return;
    }

    // Show main menu
    Sleep(1500);
    Screen->Cursor = crDefault;
    FSKShowMenuForm();
    Hide();
}

// FilmShrink about method
void TFSKSplashForm::FSKAbout(TForm *Form)
{
    ParentForm = Form;
    ParentForm->Enabled = false;

    // Fade in
    Repaint();
    for (int i = 1; i < 256; i += 4)
        AlphaBlendValue = i;
    AlphaBlendValue = 255;
    Repaint();
}


