/*=============================================================================
    Project Name:
            FilmShrink

	Module Name:
			FSKTemplate.cpp

	Abstract:
			FilmShrink templates and configuration

    License:
            GNU General Public License (GPL)

	Revision History:
			* Nov 13, 2004:  Created by Attila Afra
            * Feb  6, 2005:  Minor changes
            * Feb 17, 2005:  Loss protection support
            * Feb 22, 2005:  Advanced video codec params

------------------------------------------------------------------------------
	Copyright  2004-2005 Attila T. Afra. All rights reserved.
=============================================================================*/

// Warnings
#pragma warn -8060  // Possibly incorrect assignment

// Includes
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "FSKTemplate.h"

/*-----------------------------------------------------------------------------
    RealProducer Job File Template
-----------------------------------------------------------------------------*/

// .rpjf Body Main Part
//
// Paramaters: enableTwoPass[s], Title[s], Author[s], Copyright[s], Keywords[s],
//             Description[s], Content Rating[d],
//             avFileInput[s], fileDestination[s], destinationRollSize[d], disableAudio[s],
//             avgBitrate[d], maxBitrate[d]

#define RPJF_TEMPLATE_BEGIN "\
<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n\
<job xmlns=\"http:\/\/ns.real.com/tools/job.2.0\" xmlns:xsi=\"http:\/\/www.w3.org/2001/XMLSchema-instance\"  xsi:schemaLocation=\"http:\/\/ns.real.com/tools/job.2.0 http:\/\/ns.real.com/tools/job.2.0.xsd\">\n\
  <enableTwoPass type=\"bool\">%s</enableTwoPass>\n\
  <clipInfo>\n\
    <entry>\n\
      <name>Title</name>\n\
      <value type=\"string\">%s</value>\n\
    </entry>\n\
    <entry>\n\
      <name>Author</name>\n\
      <value type=\"string\">%s</value>\n\
    </entry>\n\
    <entry>\n\
      <name>Copyright</name>\n\
      <value type=\"string\">%s</value>\n\
    </entry>\n\
    <entry>\n\
      <name>Keywords</name>\n\
      <value type=\"string\">%s</value>\n\
    </entry>\n\
    <entry>\n\
      <name>Description</name>\n\
      <value type=\"string\">%s</value>\n\
    </entry>\n\
    <entry>\n\
      <name>Content Rating</name>\n\
      <value type=\"uint\">%d</value>\n\
    </entry>\n\
  </clipInfo>\n\
  <inputs>\n\
    <input xsi:type=\"avFileInput\">\n\
      <filename type=\"string\">%s</filename>\n\
      <pluginName type=\"string\">rn-avfile-directshow</pluginName>\n\
      <prefilters>\n\
        <prefilter xsi:type=\"deinterlacePrefilter\">\n\
          <pluginName type=\"string\">rn-prefilter-deinterlace</pluginName>\n\
          <enabled type=\"bool\">false</enabled>\n\
          <manual type=\"bool\">true</manual>\n\
          <deinterlace type=\"bool\">false</deinterlace>\n\
          <inverseTelecine type=\"bool\">false</inverseTelecine>\n\
        </prefilter>\n\
      </prefilters>\n\
    </input>\n\
  </inputs>\n\
  <parOutputs>\n\
    <output>\n\
      <destinations>\n\
        <destination xsi:type=\"fileDestination\">\n\
          <name type=\"string\">FileOutput</name>\n\
          <filename type=\"string\">%s</filename>\n\
          <destinationRollSize type=\"uint\">%d</destinationRollSize>\n\
        </destination>\n\
      </destinations>\n\
      <mediaProfile>\n\
        <audienceRefs>\n\
          <audienceRef>FilmShrink</audienceRef>\n\
        </audienceRefs>\n\
        <audioMode type=\"string\">music</audioMode>\n\
        <videoMode type=\"string\">normal</videoMode>\n\
        <disableAudio type=\"bool\">%s</disableAudio>\n\
        <disableVideo type=\"bool\">false</disableVideo>\n\
        <outputWidth type=\"uint\">0</outputWidth>\n\
        <outputHeight type=\"uint\">0</outputHeight>\n\
        <resizeQuality type=\"string\">high</resizeQuality>\n\
        <audioResamplingQuality type=\"string\">high</audioResamplingQuality>\n\
      </mediaProfile>\n\
    </output>\n\
  </parOutputs>\n\
  <audiences>\n\
    <audience>\n\
      <name type=\"string\">FilmShrink</name>\n\
      <avgBitrate type=\"uint\">%d</avgBitrate>\n\
      <maxBitrate type=\"uint\">%d</maxBitrate>\n\
      <streams>\n\
"


// .rpjf Video Part (old RC)
//
// Parameters: encoderComplexity[d],
//             comment[s], firstPassComplexity[d], comment[s],
//             enableRPR[s],
//             comment[s], maxConsecutiveBFrames[d], comment[s],
//             comment[s], patternAdaptivity[d], comment[s],
//             inloopCutOffQuant[d],
//             inloopCutOffCompatible[s], inloopCutOffBUseRefQuant[s],
//             chromaModeDecision[s], encodeAllFrames[s],
//             encodingType[s], quality[d],
//             maxFrameRate[f],
//             enableLossProtection[s]

#define RPJF_TEMPLATE_VIDEO_OLDRC "\
        <stream xsi:type=\"videoStream\">\n\
          <pluginName type=\"string\">rn-videocodec-realvideo</pluginName>\n\
          <codecName type=\"string\">rv10</codecName>\n\
          <codecProperties type=\"bag\">\n\
            <encoderComplexity type=\"uint\">%d</encoderComplexity>\n\
            %s<firstPassComplexity type=\"uint\">%d</firstPassComplexity>%s\n\
            <enableRPR type=\"bool\">%s</enableRPR>\n\
            <noisyEdgeFilter type=\"bool\">false</noisyEdgeFilter>\n\
            <rcEnableCurveCompression type=\"bool\">false</rcEnableCurveCompression>\n\
            %s<maxConsecutiveBFrames type=\"uint\">%d</maxConsecutiveBFrames>%s\n\
            %s<patternAdaptivity type=\"uint\">%d</patternAdaptivity>%s\n\
            <inloopCutOffQuant type=\"uint\">%d</inloopCutOffQuant>\n\
            <inloopCutOffCompatible type=\"bool\">%s</inloopCutOffCompatible>\n\
            <inloopCutOffBUseRefQuant type=\"bool\">%s</inloopCutOffBUseRefQuant>\n\
            <chromaModeDecision type=\"bool\">%s</chromaModeDecision>\n\
            <encodeAllFrames type=\"bool\">%s</encodeAllFrames>\n\
          </codecProperties>\n\
          <encodingType type=\"string\">%s</encodingType>\n\
          <quality type=\"uint\">%d</quality>\n\
          <maxStartupLatency type=\"double\">4</maxStartupLatency>\n\
          <maxFrameRate type=\"double\">%.3f</maxFrameRate>\n\
          <maxKeyFrameInterval type=\"double\">10</maxKeyFrameInterval>\n\
          <enableLossProtection type=\"bool\">%s</enableLossProtection>\n\
        </stream>\n\
"


// .rpjf Video Part (new RC)
//
// Parameters: encoderComplexity[d], firstPassComplexity[d],
//             enableRPR[s],
//             rcSourceFrameRate[d], 
//             rcHighBitrateReduce[d], rcLowBitrateBoost[d],
//             rcPFrameRefQuant[d], rcBFrameRefQuant[d],
//             comment[s], maxConsecutiveBFrames[d], comment[s],
//             comment[s], patternAdaptivity[d], comment[s],
//             inloopCutOffQuant[d],
//             inloopCutOffCompatible[s], inloopCutOffBUseRefQuant[s],
//             chromaModeDecision[s],
//             maxFrameRate[f],
//             enableLossProtection[s]

#define RPJF_TEMPLATE_VIDEO_NEWRC "\
        <stream xsi:type=\"videoStream\">\n\
          <pluginName type=\"string\">rn-videocodec-realvideo</pluginName>\n\
          <codecName type=\"string\">rv10</codecName>\n\
          <codecProperties type=\"bag\">\n\
            <encoderComplexity type=\"uint\">%d</encoderComplexity>\n\
            <firstPassComplexity type=\"uint\">%d</firstPassComplexity>\n\
            <enableRPR type=\"bool\">%s</enableRPR>\n\
            <noisyEdgeFilter type=\"bool\">false</noisyEdgeFilter>\n\
            <rcEnableCurveCompression type=\"bool\">true</rcEnableCurveCompression>\n\
            <rcSourceFrameRate type=\"uint\">%d</rcSourceFrameRate>\n\
            <rcHighBitrateReduce type=\"uint\">%d</rcHighBitrateReduce>\n\
            <rcLowBitrateBoost type=\"uint\">%d</rcLowBitrateBoost>\n\
            <rcPFrameRefQuant type=\"uint\">%d</rcPFrameRefQuant>\n\
            <rcBFrameRefQuant type=\"uint\">%d</rcBFrameRefQuant>\n\
            %s<maxConsecutiveBFrames type=\"uint\">%d</maxConsecutiveBFrames>%s\n\
            %s<patternAdaptivity type=\"uint\">%d</patternAdaptivity>%s\n\
            <inloopCutOffQuant type=\"uint\">%d</inloopCutOffQuant>\n\
            <inloopCutOffCompatible type=\"bool\">%s</inloopCutOffCompatible>\n\
            <inloopCutOffBUseRefQuant type=\"bool\">%s</inloopCutOffBUseRefQuant>\n\
            <chromaModeDecision type=\"bool\">%s</chromaModeDecision>\n\
          </codecProperties>\n\
          <encodingType type=\"string\">vbrBitrate</encodingType>\n\
          <quality type=\"uint\">100</quality>\n\
          <maxStartupLatency type=\"double\">4</maxStartupLatency>\n\
          <maxFrameRate type=\"double\">%.3f</maxFrameRate>\n\
          <maxKeyFrameInterval type=\"double\">10</maxKeyFrameInterval>\n\
          <enableLossProtection type=\"bool\">%s</enableLossProtection>\n\
        </stream>\n\
"


// .rpjf Audio Part
//
// Parameters: codecName[s], codecFlavor[d]

#define RPJF_TEMPLATE_AUDIO "\
        <stream xsi:type=\"audioStream\">\n\
          <pluginName type=\"string\">rn-audiocodec-realaudio</pluginName>\n\
          <codecName type=\"string\">%s</codecName>\n\
          <codecFlavor type=\"uint\">%d</codecFlavor>\n\
          <encodingComplexity type=\"string\">high</encodingComplexity>\n\
          <streamContext type=\"bag\">\n\
            <presentationType type=\"string\">audio-video</presentationType>\n\
            <audioMode type=\"string\">music</audioMode>\n\
          </streamContext>\n\
        </stream>\n\
"


// .rpjf Body End Part

#define RPJF_TEMPLATE_END "\
      </streams>\n\
    </audience>\n\
  </audiences>\n\
</job>\n\
"

/*---------------------------------------------------------------------------*/

// .rpjf Writer Function
bool FSKWriteRPJF(const char *Filename,

                  // Clip info
                  char *Title, char *Author, char *Copyright, char *Keywords,
                  char *Desc, int Rating,
                  char *Input, char *Output,

                  // General
                  bool  enableTwoPass,
                  int   destinationRollSize,
                  int   avgBitrate, int maxBitrate,

                  // Video
                  int   encoderComplexity, int firstPassComplexity,
                  bool  enableRPR,
                  bool  rcEnableCurveCompression,
                  int   rcHighBitrateReduce, int rcLowBitrateBoost,
                  int   rcPFrameRefQuant, int rcBFrameRefQuant,
                  bool  AdaptiveB,
                  int   maxConsecutiveBFrames, int patternAdaptivity,
                  int   inloopCutOffQuant,
                  bool  inloopCutOffCompatible, bool inloopCutOffBUseRefQuant,
                  bool  chromaModeDecision, bool encodeAllFrames,
                  int   quality,
                  float FrameRate,
                  bool  enableLossProtection,

                  // Audio
                  bool  Audio,
                  char *codecName, int codecFlavor)
{
    const char _false[] = "false", _true[] = "true",
               _comment1[] = "<!-- ", _comment2[] = " -->", _null[] = "";

    FILE *File;
    const char *_enableTwoPass, *_disableAudio, *_enableRPR,
               *_inloopCutOffCompatible, *_inloopCutOffBUseRefQuant,
               *_chromaModeDecision, *_encodeAllFrames, *_enableLossProtection,
               *_firstPassComplexity_comment1, *_firstPassComplexity_comment2,
               *_maxConsecutiveBFrames_comment1, *_maxConsecutiveBFrames_comment2,
               *_patternAdaptivity_comment1, *_patternAdaptivity_comment2;
    char _encodingType[256];

    // Open file
    File = fopen(Filename, "wt");
    if (!File)
        return false;

    // Bools
    if (Audio)
        _disableAudio = _false;
    else
        _disableAudio = _true;

    if (enableTwoPass)
    {
        _enableTwoPass = _true;

        _firstPassComplexity_comment1 = _null;
        _firstPassComplexity_comment2 = _null;
    }
    else
    {
        _enableTwoPass = _false;

        _firstPassComplexity_comment1 = _comment1;
        _firstPassComplexity_comment2 = _comment2;
    }

    if (!rcEnableCurveCompression)
    {
        if (quality >= 0)
            strcpy(_encodingType, "vbrQuality");
        else
        {
            strcpy(_encodingType, "vbrBitrate");
            quality = 100;
        }
    }

    if (enableRPR)
        _enableRPR = _true;
    else
        _enableRPR = _false;

    if (inloopCutOffCompatible)
        _inloopCutOffCompatible = _true;
    else
        _inloopCutOffCompatible = _false;

    if (inloopCutOffBUseRefQuant)
        _inloopCutOffBUseRefQuant = _true;
    else
        _inloopCutOffBUseRefQuant = _false;

    if (chromaModeDecision)
        _chromaModeDecision = _true;
    else
        _chromaModeDecision = _false;

    if (encodeAllFrames)
        _encodeAllFrames = _true;
    else
        _encodeAllFrames = _false;

    if (enableLossProtection)
        _enableLossProtection = _true;
    else
        _enableLossProtection = _false;

    if (AdaptiveB)
    {
        _maxConsecutiveBFrames_comment1 = _null;
        _maxConsecutiveBFrames_comment2 = _null;
        _patternAdaptivity_comment1     = _comment1;
        _patternAdaptivity_comment2     = _comment2;
    }
    else
    {
        _maxConsecutiveBFrames_comment1 = _comment1;
        _maxConsecutiveBFrames_comment2 = _comment2;
        _patternAdaptivity_comment1     = _null;
        _patternAdaptivity_comment2     = _null;
    }

    // Begin
    fprintf(File, RPJF_TEMPLATE_BEGIN,
            _enableTwoPass, Title, Author, Copyright, Keywords, Desc, Rating,
            Input, Output, destinationRollSize, _disableAudio,
            avgBitrate, maxBitrate);

    // Video
    if (rcEnableCurveCompression)
        fprintf(File, RPJF_TEMPLATE_VIDEO_NEWRC,
                encoderComplexity, firstPassComplexity,
                _enableRPR,
                int(FrameRate * 1000),
                rcHighBitrateReduce, rcLowBitrateBoost,
                rcPFrameRefQuant, rcBFrameRefQuant,
                _maxConsecutiveBFrames_comment1, maxConsecutiveBFrames, _maxConsecutiveBFrames_comment2,
                _patternAdaptivity_comment1, patternAdaptivity, _patternAdaptivity_comment2,
                inloopCutOffQuant,
                _inloopCutOffCompatible, _inloopCutOffBUseRefQuant,
                _chromaModeDecision,
                ceil(FrameRate),
                _enableLossProtection);
    else
        fprintf(File, RPJF_TEMPLATE_VIDEO_OLDRC,
                encoderComplexity,
                _firstPassComplexity_comment1, firstPassComplexity, _firstPassComplexity_comment2,
                _enableRPR,
                _maxConsecutiveBFrames_comment1, maxConsecutiveBFrames, _maxConsecutiveBFrames_comment2,
                _patternAdaptivity_comment1, patternAdaptivity, _patternAdaptivity_comment2,
                inloopCutOffQuant,
                _inloopCutOffCompatible, _inloopCutOffBUseRefQuant,
                _chromaModeDecision, _encodeAllFrames,
                _encodingType, quality,
                ceil(FrameRate),
                _enableLossProtection);

    // Audio
    if (Audio)
        fprintf(File, RPJF_TEMPLATE_AUDIO, codecName, codecFlavor);

    // End
    fprintf(File, RPJF_TEMPLATE_END);

    // Close file
    fclose(File);
    return true;
}

/*---------------------------------------------------------------------------*/

// Get the video complexity from an RPJF file
bool FSKGetRPJFVideoComplexity(const char *Filename, bool& TwoPass, int& VideoEncComplexity, int& VideoPass1Complexity)
{
    FILE *File;         // file handle
    char *Buffer, *p;   // data buffer and pointer
    int   Length;       // length of the buffer

    // Open file
    if ((File = fopen(Filename, "rt")) == NULL)
        return false;

    // Read file
    fseek(File, 0, SEEK_END);
    Length = ftell(File);
    Buffer = new char[Length + 1];
    fseek(File, 0, SEEK_SET);
    fread(Buffer, Length, 1, File);
    Buffer[Length] = 0;

    // Close file
    fclose(File);

    // Verify file
    if (!strstr(Buffer, "<job xmlns=\"http://ns.real.com/tools/job.2.0\""))
    {
        delete[] Buffer;
        return false;
    }

    // Get number of passes
    TwoPass = strstr(Buffer, "<enableTwoPass type=\"bool\">true</enableTwoPass>") != NULL;

    // Get encoding complexity
    if (p = strstr(Buffer, "<encoderComplexity type=\"uint\">"))
    {
        if (sscanf(p, "<encoderComplexity type=\"uint\">%d</encoderComplexity>", &VideoEncComplexity) != 1)
            VideoEncComplexity = -1;
    }
    else
        VideoEncComplexity = -1;

    // Get first pass complexity
    if (p = strstr(Buffer, "<firstPassComplexity type=\"uint\">"))
    {
        if (sscanf(p, "<firstPassComplexity type=\"uint\">%d</firstPassComplexity>", &VideoPass1Complexity) != 1)
            VideoPass1Complexity = -1;
    }
    else
        VideoPass1Complexity = -1;

    // Free memory
    delete[] Buffer;

    // Success
    return true;
}

// Get relative video compression speed
int FSKVideoEHQSpeed(int EHQ)
{
    if (EHQ < 60)
        return FSK_EHQ_50;
    else
        if (EHQ < 70)
            return FSK_EHQ_60;
        else
            if (EHQ < 78)
                return FSK_EHQ_70;
            else
                if (EHQ < 85)
                    return FSK_EHQ_80;
                else
                    if (EHQ < 95)
                        return FSK_EHQ_90;
                    else
                        return FSK_EHQ_100;
}
